# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
# Copyright 2012 Canonical
# Author: Thomi Richards
#
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 3, as published
# by the Free Software Foundation.

"""Autopilot test case class for Unity-specific tests."""



from codecs import open
from autopilot.introspection import get_proxy_object_for_existing_process
from autopilot.matchers import Eventually
from autopilot.testcase import AutopilotTestCase
from dbus import DBusException
from logging import getLogger
import os
import sys
from tempfile import mktemp
from time import sleep
try:
    import windowmocker
    import json
    HAVE_WINDOWMOCKER=True
except ImportError:
    HAVE_WINDOWMOCKER=False
from subprocess import check_output
import time
import tempfile
from testtools.content import text_content
from testtools.matchers import Equals
from unittest.case import SkipTest

from unity.emulators import ensure_unity_is_running
from unity.emulators.workspace import WorkspaceManager
from unity.emulators.compiz import get_compiz_setting, get_global_context
from unity.emulators.unity import (
    set_log_severity,
    start_log_to_file,
    reset_logging,
    Unity
    )
from unity.emulators.X11 import reset_display

from Xlib import display
from Xlib import Xutil

from gi.repository import GLib, Gio

log = getLogger(__name__)


class UnityTestCase(AutopilotTestCase):
    """Unity test case base class, with improvments specific to Unity tests."""

    def setUp(self):
        super(UnityTestCase, self).setUp()
        try:
            ensure_unity_is_running()
        except RuntimeError:
            log.error("Unity doesn't appear to be running, exiting.")
            sys.exit(1)

        self._unity = get_proxy_object_for_existing_process(
            connection_name=Unity.DBUS_SERVICE,
            object_path=Unity.DBUS_OBJECT,
            emulator_base=Unity
        )

        self._setUpUnityLogging()
        self._initial_workspace_num = self.workspace.current_workspace
        self.addCleanup(self.check_test_behavior)
        #
        # Setting this here since the show desktop feature seems to be a bit
        # ropey. Once it's been proven to work reliably we can remove this line:
        self.set_unity_log_level("unity.wm.compiz", "DEBUG")

        # For the length of the test, disable screen locking
        self._desktop_settings = Gio.Settings.new("org.gnome.desktop.lockdown")
        lock_state = self._desktop_settings.get_boolean("disable-lock-screen")
        self._desktop_settings.set_boolean("disable-lock-screen", True)
        self.addCleanup(self._desktop_settings.set_boolean, "disable-lock-screen", lock_state)

    def check_test_behavior(self):
        """Fail the test if it did something naughty.

        This includes leaving the dash or the hud open, changing the current
        workspace, or leaving the system in show_desktop mode.

        """
        well_behaved = True
        reasons = []
        log.info("Checking system state for badly behaving test...")

        # Have we switched workspace?
        if not self.well_behaved(self.workspace, current_workspace=self._initial_workspace_num):
            well_behaved = False
            reasons.append("The test changed the active workspace from %d to %d." \
                % (self._initial_workspace_num, self.workspace.current_workspace))
            log.warning("Test changed the active workspace, changing it back...")
            self.workspace.switch_to(self._initial_workspace_num)
        # Have we left the dash open?
        if not self.well_behaved(self.unity.dash, visible=False):
            well_behaved = False
            reasons.append("The test left the dash open.")
            log.warning("Test left the dash open, closing it...")
            self.unity.dash.ensure_hidden()
        # ... or the hud?
        if not self.well_behaved(self.unity.hud, visible=False):
            well_behaved = False
            reasons.append("The test left the hud open.")
            log.warning("Test left the hud open, closing it...")
            self.unity.hud.ensure_hidden()
        # Are we in show desktop mode?
        if not self.well_behaved(self.unity.window_manager, showdesktop_active=False):
            well_behaved = False
            reasons.append("The test left the system in show_desktop mode.")
            log.warning("Test left the system in show desktop mode, exiting it...")
            # It is not possible to leave show desktop mode if there are no
            # app windows. So, just open a window and perform the show
            # desktop action until the desired state is acheived, then close
            # the window. The showdesktop_active state will persist.
            #
            # In the event that this doesn't work, wait_for will throw an
            # exception.
            win = self.process_manager.start_app_window('Calculator', locale='C')
            self.keybinding("window/show_desktop")
            count = 1
            while self.unity.window_manager.showdesktop_active:
                self.keybinding("window/show_desktop")
                sleep(count)
                count+=1
                if count > 10:
                    break
            win.close()
            self.unity.window_manager.showdesktop_active.wait_for(False)
        for launcher in self.unity.launcher.get_launchers():
            if not self.well_behaved(launcher, in_keynav_mode=False):
                well_behaved = False
                reasons.append("The test left the launcher keynav mode enabled.")
                log.warning("Test left the launcher in keynav mode, exiting it...")
                launcher.key_nav_cancel()
            if not self.well_behaved(launcher, in_switcher_mode=False):
                well_behaved = False
                reasons.append("The test left the launcher in switcher mode.")
                log.warning("Test left the launcher in switcher mode, exiting it...")
                launcher.switcher_cancel()
            if not self.well_behaved(launcher, quicklist_open=False):
                well_behaved = False
                reasons.append("The test left a quicklist open.")
                log.warning("The test left a quicklist open.")
                self.keyboard.press_and_release('Escape')

        if not well_behaved:
            self.fail("/n".join(reasons))
        else:
            log.info("Test was well behaved.")

    def well_behaved(self, object, **kwargs):
        try:
            self.assertProperty(object, **kwargs)
        except AssertionError:
            return False
        return True

    @property
    def unity(self):
        return self._unity

    @property
    def workspace(self):
        if not getattr(self, '__workspace', None):
            self.__workspace = WorkspaceManager()
        return self.__workspace

    def _setUpUnityLogging(self):
        self._unity_log_file_name = mktemp(prefix=self.shortDescription())
        start_log_to_file(self._unity_log_file_name)
        self.addCleanup(self._tearDownUnityLogging)

    def _tearDownUnityLogging(self):
        # If unity dies, our dbus interface has gone, and reset_logging will fail
        # but we still want our log, so we ignore any errors.
        try:
            reset_logging()
        except DBusException:
            pass
        with open(self._unity_log_file_name, encoding='utf-8') as unity_log:
            self.addDetail('unity-log', text_content(unity_log.read()))
        os.remove(self._unity_log_file_name)
        self._unity_log_file_name = ""

    def set_unity_log_level(self, component, level):
        """Set the unity log level for 'component' to 'level'.

        Valid levels are: TRACE, DEBUG, INFO, WARNING and ERROR.

        Components are dotted unity component names. The empty string specifies
        the root logging component.
        """
        valid_levels = ('TRACE', 'DEBUG', 'INFO', 'WARN', 'WARNING', 'ERROR')
        if level not in valid_levels:
            raise ValueError("Log level '%s' must be one of: %r" % (level, valid_levels))
        set_log_severity(component, level)

    def assertNumberWinsIsEventually(self, app, num):
        """Asserts that 'app' eventually has 'num' wins. Waits up to 10 seconds."""

        self.assertThat(lambda: len(app.get_windows()), Eventually(Equals(num)))

    def launch_test_window(self, window_spec={}):
        """Launch a test window, for the duration of this test only.

        This uses the 'window-mocker' application, which is not part of the
        python-autopilot or unity-autopilot packages. To use this method, you
        must have python-windowmocker installed. If the package is not installed,
        this method will raise a SkipTest exception, causing the calling test
        to be silently skipped.

        window_spec is a list or dictionary that conforms to the window-mocker
        specification.

        """
        if not HAVE_WINDOWMOCKER:
            raise SkipTest("The python-windowmocker package is required to run this test.")

        if 'Window Mocker' not in self.process_manager.KNOWN_APPS:
            self.process_manager.register_known_application(
                'Window Mocker',
                'window-mocker.desktop',
                'window-mocker'
                )
        if window_spec:
            file_path = tempfile.mktemp()
            json.dump(window_spec, open(file_path, 'w'))
            self.addCleanup(os.remove, file_path)
            return self.process_manager.start_app_window('Window Mocker', [file_path])
        else:
            return self.process_manager.start_app_window('Window Mocker')

    def close_all_windows(self, application_name):
        for w in self.process_manager.get_open_windows_by_application(application_name):
            w.close()

        self.assertThat(lambda: len(self.process_manager.get_open_windows_by_application(application_name)), Eventually(Equals(0)))

    def register_nautilus(self):
        self.addCleanup(self.process_manager.unregister_known_application, "Nautilus")
        self.process_manager.register_known_application("Nautilus", "org.gnome.Nautilus.desktop", "nautilus")

    def get_startup_notification_timestamp(self, bamf_window):
        atom = display.Display().intern_atom('_NET_WM_USER_TIME')
        atom_type = display.Display().intern_atom('CARDINAL')
        return bamf_window.x_win.get_property(atom, atom_type, 0, 1024).value[0]

    def call_gsettings_cmd(self, command, schema, key, value=None):
        """Set a desktop wide gsettings option
        """
        settings = Gio.Settings.new(schema)

        if command == "get":
            return settings.get_value(key).print_(type_annotate=False)
        elif command == "set":
            settings.set_value(key, GLib.Variant.parse(type=None, text=value))
            settings.apply()
            reset_display()
        elif command == "reset":
            settings.reset(key)

    def set_unity_option(self, option_name, option_value):
        """Set an option in the unity compiz plugin options.

        .. note:: The value will be set for the current test only, and
         automatically undone when the test ends.

        :param option_name: The name of the unity option.
        :param option_value: The value you want to set.
        :raises: **KeyError** if the option named does not exist.

        """
        self.set_compiz_option("unityshell", option_name, option_value)

    def set_compiz_option(self, plugin_name, option_name, option_value):
        """Set a compiz option for the duration of this test only.

        .. note:: The value will be set for the current test only, and
         automatically undone when the test ends.

        :param plugin_name: The name of the compiz plugin where the option is
         registered. If the option is not in a plugin, the string "core" should
         be used as the plugin name.
        :param option_name: The name of the unity option.
        :param option_value: The value you want to set.
        :raises: **KeyError** if the option named does not exist.

        """
        old_value = self._set_compiz_option(plugin_name, option_name, option_value)
        # Cleanup is LIFO, during clean-up also allow unity to respond
        self.addCleanup(time.sleep, 0.5)
        self.addCleanup(self._set_compiz_option, plugin_name, option_name, old_value)
        # Allow unity time to respond to the new setting.
        time.sleep(0.5)

    def _set_compiz_option(self, plugin_name, option_name, option_value):
        log.info("Setting compiz option '%s' in plugin '%s' to %r",
            option_name, plugin_name, option_value)
        setting = get_compiz_setting(plugin_name, option_name)
        old_value = setting.Value
        setting.Value = option_value
        get_global_context().Write()
        return old_value



#!/usr/bin/env python3

"""
This file is part of "blighty" and "uwidgets" which is released under GPL.

See file LICENCE or go to http://www.gnu.org/licenses/ for full license
details.

uwidgets is a desktop widget creation and management library for Python 3.

Copyright (c) 2022 Rudra Saraswat <rs2009@ubuntu.com>.
Copyright (c) 2018 Gabriele N. Tornetta <phoenix1987@gmail.com>.
All rights reserved.

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.
You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""

from uwidgets import CanvasGravity
from uwidgets.x11 import Canvas, start_event_loop
from uwidgets.settings import UnityWallpaper

import cairo
import subprocess
import configparser

from gi.repository import GLib


class Fonts:
    UBUNTU_NORMAL = "Ubuntu", cairo.FontSlant.NORMAL, cairo.FontWeight.NORMAL


class UnBackground(Canvas):
    def on_button_pressed(self, button, state, x, y):
        if button == 1:    # Left button
            self.change_wallpaper()
        elif button == 3:  # Right button
            subprocess.run(['unity-control-center', 'appearance'])

    def change_wallpaper(ctx):
        UnityWallpaper().set_wallpaper_from_url("https://source.unsplash.com/random/3840x2160")

    def draw_background(ctx):
        size = ctx.canvas.get_size()
        ctx.rectangle(0, 0, *size)
        ctx.set_source_rgba(0, 0, 0, 0.6)
        ctx.fill()

    def draw_decoration(ctx):
        ctx.rectangle(0, 0, ctx.canvas.width, 3)
        ctx.set_source_rgba(1, 1, 1, 1)
        ctx.fill()

    def draw_info(ctx):
        ctx.set_source_rgb(0.9, 0.9, 0.9)

        ctx.select_font_face(*Fonts.UBUNTU_NORMAL)
        ctx.set_font_size(24)
        ctx.move_to(20, 42)
        ctx.show_text('Unsplash | Random Background')

        ctx.set_source_rgb(0.5, 0.5, 0.5)
        ctx.set_font_size(15)
        ctx.move_to(20, 64)
        ctx.show_text('Click this to set a random Unsplash background (requires internet).')

    def on_draw(self, ctx):
        ctx.draw_background()
        ctx.draw_decoration()
        ctx.draw_info()


if __name__ == "__main__":
    config = configparser.ConfigParser()
    config.read('settings.ini')
    unbg = UnBackground(
        x = int(config.get('settings', 'margin_x')),
        y = int(config.get('settings', 'margin_y')),
        width = 540,
        height = 80,
        gravity = getattr(CanvasGravity, config.get('settings', 'gravity')),
        interval = 1000
    )

    unbg.show()
    start_event_loop()



# This file is part of "blighty" and "uwidgets" which is released under GPL.
#
# See file LICENCE or go to http://www.gnu.org/licenses/ for full license
# details.
#
# uwidgets is a desktop widget creation and management library for Python 3.
#
# Copyright (c) 2018 Gabriele N. Tornetta <phoenix1987@gmail.com>.
# All rights reserved.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""
This module contains the common objects and types for the different kind of
canvases provided by ``uwidgets``.
"""

# XWayland fix
import os
os.environ["GDK_BACKEND"] = "x11"

from . _extended_context import ExtendedContext

from . _brush import brush, TextAlign


class CanvasType:
    """The Canvas type.

    The canvas types enumerated in this Python type reflect the same window
    types that one can request to the window manager via the `Extended
    Window Manager Hints <https://standards.freedesktop.org/wm-spec/wm-spec-1.3.html>`_.

    - ``NORMAL`` is a normal top-level window.
    - ``DESKTOP`` is a window drawn directly on the desktop.
    - ``DOCK`` indicates a dock or panel window that will usually stay on top
      of other windows.
    - ``UNDECORATED`` is a type of window that behaves as a toolbar. As such,
      it is undecorated.
    """

    NORMAL = 0         # _NET_WM_WINDOW_TYPE_NORMAL
    DESKTOP = 1        # _NET_WM_WINDOW_TYPE_DESKTOP
    DOCK = 2           # _NET_WM_WINDOW_TYPE_DOCK
    UNDECORATED = 3    # _NET_WM_WINDOW_TYPE_TOOLBAR


class CanvasGravity:
    """Window gravity control type.

    The positioning of a canvas on the screen is controlled by its gravity.
    By default, a window is positioned in a coordinate system where the origin
    is located in the top-left corner of the screen, with the *x* axis running
    horizontally from left to right, and the *y* from top to bottom. To change
    the location of the origin, use one of the following values.
    """

    NORTH_WEST = 1
    NORTH = 2
    NORTH_EAST = 3
    WEST = 4
    CENTER = 5
    EAST = 6
    SOUTH_WEST = 7
    SOUTH = 8
    SOUTH_EAST = 9
    STATIC = 10



#!/usr/bin/python

from distutils.core import setup
from setuptools import find_packages

setup(
    name='unity',
    version='1.0',
    description='Unity autopilot tests.',
    author='Alex Launi',
    author_email='alex.launi@canonical.com',
    url='https://launchpad.net/unity',
    license='GPLv3',
    packages=find_packages(),
)



# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
# Copyright 2012 Canonical
# Authors: Thomi Richards,
#          Marco Trevisan (Treviño)
#
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 3, as published
# by the Free Software Foundation.


from autopilot.matchers import Eventually
import logging
from testtools.matchers import Equals, LessThan, GreaterThan
from testtools import skip
from time import sleep

from unity.tests import UnityTestCase

logger = logging.getLogger(__name__)


class LauncherCaptureTests(UnityTestCase):
    """Test the launchers ability to capture/not capture the mouse."""

    def setUp(self):
        super(LauncherCaptureTests, self).setUp()

        if self.display.get_num_screens() <= 1:
            self.skipTest("This test requires two or more monitors.")

        self.set_unity_option('launcher_capture_mouse', True)
        self.set_unity_option('num_launchers', 0)
        self.setHideMode(0)

    def setHideMode(self, mode):
        self.set_unity_option('launcher_hide_mode', mode)
        launcher = self.unity.launcher.get_launcher_for_monitor(0)
        self.assertThat(launcher.hidemode, Eventually(Equals(mode)))

    def leftMostMonitor(self):
        x1, y1, width, height = self.display.get_screen_geometry(0)
        x2, y2, width, height = self.display.get_screen_geometry(1)

        if x1 < x2:
            return 0
        return 1

    def rightMostMonitor(self):
        # TODO: This will break setups with 3 or more monitors.
        return 1 - self.leftMostMonitor()


    @skip("Test is no longer possible due to changes in AP mouse movement.")
    def test_launcher_captures_while_sticky_and_revealed(self):
        """Tests that the launcher captures the mouse when moving between monitors
        while revealed.
        """
        x, y, width, height = self.display.get_screen_geometry(self.rightMostMonitor())
        self.mouse.move(x + width / 2, y + height / 2, False)
        self.mouse.move(x - width / 2, y + height / 2, True, 5, .002)

        x_fin, y_fin = self.mouse.position()
        # The launcher should have held the mouse a little bit
        self.assertThat(x_fin, GreaterThan(x - width / 2))

    def test_launcher_not_capture_while_not_sticky_and_revealed(self):
        """Tests that the launcher doesn't captures the mouse when moving between monitors
        while revealed and stick is off.
        """

        self.set_unity_option('launcher_capture_mouse', False)

        x, y, width, height = self.display.get_screen_geometry(self.rightMostMonitor())
        self.mouse.move(x + width / 2, y + height / 2, False)
        self.mouse.move(x - width / 2, y + height / 2, True, 5, .002)

        x_fin, y_fin = self.mouse.position()
        # The launcher should have held the mouse a little bit
        self.assertThat(x_fin, Equals(x - width / 2))

    @skip("Test is no longer possible due to changes in AP mouse movement.")
    def test_launcher_capture_while_not_sticky_and_hidden(self):
        """Tests that the launcher captures the mouse when moving between monitors
        while hidden and sticky is off. (moving left)
        """

        self.set_unity_option('launcher_capture_mouse', False)
        self.setHideMode(1)

        x, y, width, height = self.display.get_screen_geometry(self.rightMostMonitor())
        self.mouse.move(x + width / 2, y + height / 2, False)
        self.mouse.move(x - width / 2, y + height / 2, True, 5, .002)

        x_fin, y_fin = self.mouse.position()
        # The launcher should have held the mouse a little bit
        self.assertThat(x_fin, GreaterThan(x - width / 2))

    def test_launcher_not_capture_while_not_sticky_and_hidden_moving_right(self):
        """Tests that the launcher doesn't capture the mouse when moving between monitors
        while hidden and sticky is off (moving right).
        """

        self.set_unity_option('launcher_capture_mouse', False)
        self.setHideMode(1)

        x, y, width, height = self.display.get_screen_geometry(self.leftMostMonitor())
        self.mouse.move(x + width / 2, y + height / 2, False)
        sleep(1.5)
        self.mouse.move(x + width * 1.5, y + height / 2, True, 5, .002)

        x_fin, y_fin = self.mouse.position()
        # The launcher should have held the mouse a little bit
        self.assertThat(x_fin, Equals(x + width * 1.5))

    @skip("Test is no longer possible due to changes in AP mouse movement.")
    def test_launcher_capture_while_sticky_and_hidden_moving_right(self):
        """Tests that the launcher captures the mouse when moving between monitors
        while hidden.
        """
        self.setHideMode(1)

        x, y, width, height = self.display.get_screen_geometry(self.leftMostMonitor())
        self.mouse.move(x + width / 2, y + height / 2, False)
        sleep(1.5)
        self.mouse.move(x + width * 1.5, y + height / 2, True, 5, .002)

        x_fin, y_fin = self.mouse.position()
        # The launcher should have held the mouse a little bit
        self.assertThat(x_fin, LessThan(x + width * 1.5))




"""Functions that wrap compizconfig to avoid some unpleasantness in that module."""


from autopilot.utilities import Silence

_global_context = None

def get_global_context():
    """Get the compizconfig global context object."""
    global _global_context
    if _global_context is None:
        with Silence():
            from compizconfig import Context
            _global_context = Context()
    return _global_context


def _get_plugin(plugin_name):
    """Get a compizconfig plugin with the specified name.

    Raises KeyError of the plugin named does not exist.

    """
    ctx = get_global_context()
    with Silence():
        try:
            return ctx.Plugins[plugin_name]
        except KeyError:
            raise KeyError("Compiz plugin '%s' does not exist." % (plugin_name))


def get_compiz_setting(plugin_name, setting_name):
    """Get a compiz setting object.

    *plugin_name* is the name of the plugin (e.g. 'core' or 'unityshell')
    *setting_name* is the name of the setting (e.g. 'alt_tab_timeout')

    :raises: KeyError if the plugin or setting named does not exist.

    :returns: a compiz setting object.

    """
    plugin = _get_plugin(plugin_name)
    with Silence():
        try:
            return plugin.Screen[setting_name]
        except KeyError:
            raise KeyError("Compiz setting '%s' does not exist in plugin '%s'." % (setting_name, plugin_name))


def get_compiz_option(plugin_name, setting_name):
    """Get a compiz setting value.

    This is the same as calling:

    >>> get_compiz_setting(plugin_name, setting_name).Value

    """
    return get_compiz_setting(plugin_name, setting_name).Value




#!/usr/bin/env python

#
# Script to generate a nice PNG file of the currently running unity introspection tree.
from argparse import ArgumentParser
from os import remove
from os.path import splitext
import dbus

try:
    from autopilot.emulators.unity import get_state_by_path
except ImportError as e:
    print("Error: could not import the autopilot python module.")
    print("Make sure the autopilot module is in your $PYTHONPATH.")
    exit(1)

try:
    import pydot
except ImportError:
    print("Error: the 'pydot' module is required to run this script.")
    print("Try installing the 'python3-pydot' package.")
    exit(1)

NEXT_NODE_ID=1
NODE_BLACKLIST=["Result"]

def string_rep(dbus_type):
    """Get a string representation of various dbus types."""
    if type(dbus_type) == dbus.Boolean:
        return repr(bool(dbus_type))
    if type(dbus_type) == dbus.String:
        return dbus_type.encode('ascii', errors='ignore')
    if type(dbus_type) in (dbus.Int16, dbus.UInt16, dbus.Int32, dbus.UInt32, dbus.Int64, dbus.UInt64):
        return repr(int(dbus_type))
    if type(dbus_type) == dbus.Double:
        return repr(float(dbus_type))
    if type(dbus_type) == dbus.Array:
        return ', '.join([string_rep(i) for i in dbus_type])
    else:
        return repr(dbus_type)


def escape(s):
    """Escape a string so it can be use in a dot label."""
    return pydot.quote_if_necessary(s).replace('<','\\<').replace('>', '\\>').replace("'", "\\'")


def traverse_tree(state, parent, graph):
    """Recursively traverse state tree, building dot graph as we go."""
    global NEXT_NODE_ID
    lbl = parent.get_comment() + "|"
    # first, set labels for this node:
    bits = ["%s=%s" % (k, string_rep(state[k])) for k in sorted(state.keys()) if k != 'Children']
    lbl += "\l".join(bits)
    parent.set_label(escape('"{' + lbl + '}"'))
    if 'Children' in state:
        # Add all array nodes as children of this node.
        for child_name, child_state in state['Children']:
            if child_name in NODE_BLACKLIST:
                continue
            child = pydot.Node(str(NEXT_NODE_ID))
            NEXT_NODE_ID+=1
            child.set_comment(child_name)
            graph.add_node(child)
            graph.add_edge(pydot.Edge(parent, child))

            traverse_tree(child_state, child, graph)


if __name__ == '__main__':
    parser = ArgumentParser()
    mg = parser.add_mutually_exclusive_group(required=True)
    mg.add_argument('-o', '--output', nargs=1,
        help='Store output in specified file on disk.')
    mg.add_argument('-d','--display', action='store_true',
        help='Display output in image viewer.')

    args = parser.parse_args()

    introspection_tree = get_state_by_path('/')
    graph = pydot.Dot()
    graph.set_simplify(False)
    graph.set_node_defaults(shape='Mrecord')
    graph.set_fontname('Ubuntu')
    graph.set_fontsize('10')

    gnode_unity = pydot.Node("Unity")
    gnode_unity.set_comment("Unity")
    traverse_tree(introspection_tree[0], gnode_unity, graph)

    if args.output:
        base, extension = splitext(args.output[0])
        write_method_name = 'write_' + extension[1:]
        if hasattr(graph, write_method_name):
            getattr(graph, write_method_name)(args.output[0])
        else:
            print("Error: unsupported format: '%s'" % (extension))
    elif args.display:
        from tempfile import NamedTemporaryFile
        from subprocess import call
        tf = NamedTemporaryFile(suffix='.png', delete=False)
        tf.write(graph.create_png())
        tf.close()
        call(["eog", tf.name])
        remove(tf.name)
    else:
        print('unknown output mode!')





"""
This file is part of "blighty" and "uwidgets" which is released under GPL.

See file LICENCE or go to http://www.gnu.org/licenses/ for full license
details.

uwidgets is a desktop widget creation and management library for Python 3.

Copyright (c) 2018 Gabriele N. Tornetta <phoenix1987@gmail.com>.
All rights reserved.

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.
You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""

from . _brush import BrushSets, not_callable_from_instance


class ExtendedContext:
    """Extension of the standard `cairo.Context` class.

    This class is used to extend the vanilla `cairo.Context` with _brushes_.
    These are either methods of a subclass of `Canvas` that are prefixed with
    `draw_`, or those that are explicitly decorated with the `@brush`
    decorator.

    There shouldn't be any reasons why you'd want to use instantiate this class
    directly. The `ctx` argument that is passed to the `Canvas` `on_draw`
    callback is an instance of this class. This can be passed as an argument
    to any callable object that expects a `cairo.Context` instance. The
    underlying `Canvas` object can be accessed via the `canvas` attribute. This
    can be useful if one needs to refer to the parent canvas geometry (e.g.
    its size).
    """
    def __init__(self, ctx, canvas):
        self._ctx = ctx
        self.canvas = canvas

        collected_methods = []

        for dm in dir(canvas):
            try:
                if callable(getattr(canvas, dm)) and dm[:5] == "draw_":
                    collected_methods.append(dm)
            except RuntimeError:
                # In the GTK case, introspection breaks getattr so we ignore
                # the attributes we cannot retrieve.
                pass

        # for m in [dm for dm in dir(canvas) if callable(getattr(canvas, dm)) and dm[:5] == "draw_"]:
        for m in collected_methods:
            # Re-bind brush method and mark the original as non-callable
            setattr(self, m, getattr(type(canvas), m).__get__(self, ExtendedContext))
            setattr(canvas, m, not_callable_from_instance.__get__(canvas, type(canvas)))

        for n, m in BrushSets.get_brush_set(type(canvas).__qualname__).items():
            if n in dir(ctx):
                raise RuntimeError("Brush name '{}' clashes with attribute or method in {}".format(n, type(ctx).__qualname__))
            setattr(self, n, m.__get__(self, ExtendedContext))

    def __getattr__(self, name):
        """Access the underling context methods."""
        return getattr(self._ctx, name)



# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
# Copyright 2012 Canonical
# Author: Thomi Richards,
#         Marco Trevisan (Treviño)
#
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 3, as published
# by the Free Software Foundation.


from autopilot.display import move_mouse_to_screen
from autopilot.matchers import Eventually
from autopilot.introspection.dbus import StateNotFoundError
import os.path
from testtools.matchers import Contains, Equals, NotEquals, Not, Raises
from time import sleep
from xdg.DesktopEntry import DesktopEntry

from unity.emulators import keys
from unity.emulators.quicklist import QuicklistMenuItemLabel
from unity.emulators.launcher import LauncherPosition
from unity.tests import UnityTestCase


class QuicklistActionTests(UnityTestCase):
    """Tests for quicklist actions."""

    scenarios = [
        ('remmina', {'app_name': 'Remmina'}),
    ]

    def open_quicklist_for_icon(self, launcher_icon):
        """Open the quicklist for the given launcher icon.

        Returns the quicklist that was opened.

        """
        launcher = self.unity.launcher.get_launcher_for_monitor(0)
        launcher.click_launcher_icon(launcher_icon, button=3)
        self.addCleanup(self.keyboard.press_and_release, "Escape")
        self.assertThat(launcher_icon.get_quicklist, Eventually(NotEquals(None)))
        return launcher_icon.get_quicklist()

    def get_desktop_entry(self, application):
        # load the desktop file from disk:
        desktop_id = self.process_manager.KNOWN_APPS[application]['desktop-file']
        desktop_file = os.path.join('/usr/share/applications', desktop_id)
        return DesktopEntry(desktop_file)

    def test_quicklist_actions(self):
        """Test that all actions present in the destop file are shown in the quicklist."""
        app = self.process_manager.start_app(self.app_name)

        # get the launcher icon from the launcher:
        launcher_icon = self.unity.launcher.model.get_icon(desktop_id=app.desktop_file)
        self.assertThat(launcher_icon, NotEquals(None))

        # open the icon quicklist, and get all the text labels:
        de = self.get_desktop_entry(self.app_name)
        ql = self.open_quicklist_for_icon(launcher_icon)
        ql_item_texts = [i.text for i in ql.items if type(i) is QuicklistMenuItemLabel]

        # iterate over all the actions from the desktop file, make sure they're
        # present in the quicklist texts.
        for action in de.getActions():
            key = 'Desktop Action ' + action
            self.assertThat(de.content, Contains(key))
            name = de.get('Name', group=key, locale=True)
            self.assertThat(ql_item_texts, Contains(name))

    def test_quicklist_action_uses_startup_notification(self):
        """Tests that quicklist uses startup notification protocol."""
        self.register_nautilus()
        self.addCleanup(self.close_all_windows, "Nautilus")

        self.process_manager.start_app_window("Calculator")
        self.process_manager.start_app(self.app_name)

        nautilus_icon = self.unity.launcher.model.get_icon(desktop_id="org.gnome.Nautilus.desktop")
        ql = self.open_quicklist_for_icon(nautilus_icon)
        de = self.get_desktop_entry("Nautilus")

        new_window_action_name = de.get("Name", group="Desktop Action Window", locale=True)
        self.assertThat(new_window_action_name, NotEquals(None))
        new_win_ql_item_fn = lambda : ql.get_quicklist_item_by_text(new_window_action_name)
        self.assertThat(new_win_ql_item_fn, Eventually(NotEquals(None)))
        new_win_ql_item = new_win_ql_item_fn()

        ql.click_item(new_win_ql_item)

        nautilus_windows_fn = lambda: self.process_manager.get_open_windows_by_application("Nautilus")
        self.assertThat(lambda: len(nautilus_windows_fn()), Eventually(Equals(1)))
        [nautilus_window] = nautilus_windows_fn()

        self.assertThat(new_win_ql_item.wait_until_destroyed, Not(Raises()))

    def test_quicklist_application_item_focus_last_active_window(self):
        """This tests shows that when you activate a quicklist application item
        only the last focused instance of that application is rasied.

        This is tested by opening 2 Mahjongg and a Calculator.
        Then we activate the Calculator quicklist item.
        Then we actiavte the Mahjongg launcher icon.
        """
        char_win1 = self.process_manager.start_app_window("Character Map")
        calc_win = self.process_manager.start_app_window("Calculator")
        char_win2 = self.process_manager.start_app_window("Character Map")

        self.assertVisibleWindowStack([char_win2, calc_win, char_win1])

        char_icon = self.unity.launcher.model.get_icon(
            desktop_id=char_win1.application.desktop_file)
        calc_icon = self.unity.launcher.model.get_icon(
            desktop_id=calc_win.application.desktop_file)

        calc_ql = self.open_quicklist_for_icon(calc_icon)
        calc_ql.get_quicklist_application_item(calc_win.application.name).mouse_click()

        self.assertProperty(calc_win, is_focused=True)
        self.assertVisibleWindowStack([calc_win, char_win2, char_win1])

        char_ql = self.open_quicklist_for_icon(char_icon)
        char_ql.get_quicklist_application_item(char_win1.application.name).mouse_click()

        self.assertProperty(char_win2, is_focused=True)
        self.assertVisibleWindowStack([char_win2, calc_win, char_win1])

    def test_quicklist_application_item_initiate_spread(self):
        """This tests shows that when you activate a quicklist application item
        when an application window is focused, the spread is initiated.
        """
        char_win1 = self.process_manager.start_app_window("Character Map")
        char_win2 = self.process_manager.start_app_window("Character Map")
        char_app = char_win1.application

        self.assertVisibleWindowStack([char_win2, char_win1])
        self.assertProperty(char_win2, is_focused=True)

        char_icon = self.unity.launcher.model.get_icon(desktop_id=char_app.desktop_file)

        char_ql = self.open_quicklist_for_icon(char_icon)
        app_item = char_ql.get_quicklist_application_item(char_app.name)

        self.addCleanup(self.keybinding, "spread/cancel")
        app_item.mouse_click()
        self.assertThat(self.unity.window_manager.scale_active, Eventually(Equals(True)))
        self.assertThat(self.unity.window_manager.scale_active_for_group, Eventually(Equals(True)))

    def test_quicklist_item_triggered_closes_dash(self):
        """When any quicklist item is triggered it must close the dash."""

        calc_win = self.process_manager.start_app_window("Calculator")
        self.assertProperty(calc_win, is_focused=True)

        self.unity.dash.ensure_visible()
        self.addCleanup(self.unity.dash.ensure_hidden)

        calc_icon = self.unity.launcher.model.get_icon(
            desktop_id=calc_win.application.desktop_file)
        self.open_quicklist_for_icon(calc_icon)

        self.keyboard.press_and_release("Down")
        self.keyboard.press_and_release("Enter")
        self.assertThat(self.unity.dash.visible, Eventually(Equals(False)))

    def test_quicklist_closes_when_hud_opens(self):
        """When a quicklist is open you must still be able to open the Hud."""
        calc = self.process_manager.start_app("Calculator")

        calc_icon = self.unity.launcher.model.get_icon(desktop_id=calc.desktop_file)
        self.open_quicklist_for_icon(calc_icon)

        self.unity.hud.ensure_visible()
        self.addCleanup(self.unity.hud.ensure_hidden)
        self.assertThat(self.unity.hud.visible, Eventually(Equals(True)))

    def test_quicklist_closes_when_dash_opens(self):
        """When the quicklist is open you must still be able to open the dash."""
        calc = self.process_manager.start_app("Calculator")

        calc_icon = self.unity.launcher.model.get_icon(desktop_id=calc.desktop_file)
        self.open_quicklist_for_icon(calc_icon)

        self.unity.dash.ensure_visible()
        self.addCleanup(self.unity.dash.ensure_hidden)
        self.assertThat(self.unity.dash.visible, Eventually(Equals(True)))

    def test_right_click_opens_quicklist_if_already_open(self):
        """A right click to another icon in the launcher must
        close the current open quicklist and open the other
        icons quicklist.
        lp:890991
        """

        icons = self.unity.launcher.model.get_launcher_icons()

        icon1_ql = self.open_quicklist_for_icon(icons[1])
        self.assertThat(icon1_ql.active, Eventually(Equals(True)))

        icon0_ql = self.open_quicklist_for_icon(icons[0])
        self.assertThat(icon0_ql.active, Eventually(Equals(True)))
        self.assertThat(icon1_ql.wait_until_destroyed, Not(Raises()))

    def test_right_clicking_same_icon_doesnt_reopen_ql(self):
        """A right click to the same icon in the launcher must
        not re-open the quicklist if already open. It must hide.
        """

        calc_win = self.process_manager.start_app_window("Calculator")

        calc_icon = self.unity.launcher.model.get_icon(
            desktop_id=calc_win.application.desktop_file)

        calc_ql = self.open_quicklist_for_icon(calc_icon)
        self.assertThat(calc_ql.active, Eventually(Equals(True)))

        # We've to manually open the icon this time, as when the quicklist goes away
        # its Destroyed, so its None!
        launcher = self.unity.launcher.get_launcher_for_monitor(0)
        launcher.click_launcher_icon(calc_icon, button=3)
        self.addCleanup(self.keyboard.press_and_release, "Escape")
        calc_ql = calc_icon.get_quicklist()
        self.assertThat(calc_ql, Equals(None))


class QuicklistKeyNavigationTests(UnityTestCase):
    """Tests for the quicklist key navigation."""
    scenarios = [
        ('left', {'launcher_position': LauncherPosition.LEFT}),
        ('bottom', {'launcher_position': LauncherPosition.BOTTOM}),
    ]

    def setUp(self):
        super(QuicklistKeyNavigationTests, self).setUp()

        desktop_file = self.process_manager.KNOWN_APPS["Text Editor"]["desktop-file"]
        icon_refresh_fn = lambda : self.unity.launcher.model.get_icon(
            desktop_id=desktop_file)

        self.ql_app = self.process_manager.start_app("Text Editor")

        self.assertThat(icon_refresh_fn, Eventually(NotEquals(None)))
        self.ql_launcher_icon = icon_refresh_fn()

        self.ql_launcher = self.unity.launcher.get_launcher_for_monitor(0)

        old_pos = self.call_gsettings_cmd('get', 'com.canonical.Unity.Launcher', 'launcher-position')
        self.call_gsettings_cmd('set', 'com.canonical.Unity.Launcher', 'launcher-position', '"%s"' % self.launcher_position)
        self.addCleanup(self.call_gsettings_cmd, 'set', 'com.canonical.Unity.Launcher', 'launcher-position', old_pos)

    def open_quicklist_with_mouse(self):
        """Opens a quicklist with the mouse."""
        self.ql_launcher.click_launcher_icon(self.ql_launcher_icon, button=3)
        self.addCleanup(self.keyboard.press_and_release, "Escape")
        self.assertThat(self.ql_launcher_icon.get_quicklist,
                        Eventually(NotEquals(None)))
        self.quicklist = self.ql_launcher_icon.get_quicklist()
        self.quicklist.move_mouse_to_right()
        self.assertThat(lambda: self.quicklist.selected_item,
                        Eventually(Equals(None)))

    def open_quicklist_with_keyboard(self):
        """Opens a quicklist using the keyboard."""
        move_mouse_to_screen(0)
        self.ql_launcher.key_nav_start()
        self.addCleanup(self.ql_launcher.key_nav_cancel)

        self.ql_launcher.keyboard_select_icon(self.launcher_position, tooltip_text=self.ql_app.name)
        self.keybinding(keys[self.launcher_position + "/launcher/keynav/open-quicklist"])
        self.addCleanup(self.keybinding, "launcher/keynav/close-quicklist")

        self.assertThat(self.ql_launcher_icon.get_quicklist,
                        Eventually(NotEquals(None)))
        self.quicklist = self.ql_launcher_icon.get_quicklist()
        self.assertThat(lambda: self.quicklist.selected_item,
                        Eventually(NotEquals(None)))

    def assertCorrectItemSelected(self, item):
        """Ensure the item considers itself selected and that quicklist agrees."""
        self.assertThat(item.selected, Eventually(Equals(True)))
        self.assertThat(self.quicklist.selected_item, Equals(item))

    def test_keynav_selects_first_item_when_unselected(self):
        """Home key MUST select the first selectable item in a quicklist."""
        self.open_quicklist_with_mouse()

        self.keybinding("quicklist/keynav/first")

        expected_item = self.quicklist.selectable_items[0]
        self.assertCorrectItemSelected(expected_item)

    def test_keynav_selects_first_item_when_selected(self):
        """Home key MUST select the first selectable item in a quicklist when
        another item is selected.
        """
        self.open_quicklist_with_mouse()
        mouse_item = self.quicklist.selectable_items[-1]
        mouse_item.mouse_move_to()
        self.assertThat(mouse_item.selected, Eventually(Equals(True)))

        self.keybinding("quicklist/keynav/first")

        expected_item = self.quicklist.selectable_items[0]
        self.assertCorrectItemSelected(expected_item)

    def test_keynav_next_selects_first_item_when_unselected(self):
        """Down key MUST select the first valid item when nothing is selected."""
        self.open_quicklist_with_mouse()

        self.keybinding("quicklist/keynav/next")

        expected_item = self.quicklist.selectable_items[0]
        self.assertCorrectItemSelected(expected_item)

    def test_keynav_selects_last_item_when_unselected(self):
        """End key MUST select the last selectable item in a quicklist."""
        self.open_quicklist_with_mouse()

        self.keybinding("quicklist/keynav/last")

        expected_item = self.quicklist.selectable_items[-1]
        self.assertCorrectItemSelected(expected_item)

    def test_keynav_selects_last_item_when_selected(self):
        """End key MUST select the last selectable item in a quicklist when
        another item is selected.
        """
        self.open_quicklist_with_mouse()
        mouse_item = self.quicklist.selectable_items[0]
        mouse_item.mouse_move_to()
        self.assertThat(mouse_item.selected, Eventually(Equals(True)))

        self.keybinding("quicklist/keynav/last")

        expected_item = self.quicklist.selectable_items[-1]
        self.assertCorrectItemSelected(expected_item)

    def test_keynav_prev_selects_last_item_when_unselected(self):
        """Up key MUST select the last valid item when nothing is selected."""
        self.open_quicklist_with_mouse()

        self.keybinding("quicklist/keynav/prev")

        expected_item = self.quicklist.selectable_items[-1]
        self.assertCorrectItemSelected(expected_item)

    def test_launcher_keynav_selects_first_item(self):
        """The first selectable item of the quicklist must be selected when
        opening the quicklist using the launcher key navigation.
        """
        self.open_quicklist_with_keyboard()

        expected_item = self.quicklist.selectable_items[0]
        self.assertCorrectItemSelected(expected_item)

    def test_keynav_next_selection_works(self):
        """Down key MUST select the next valid item."""
        self.open_quicklist_with_mouse()

        for item in self.quicklist.selectable_items:
            self.keybinding("quicklist/keynav/next")
            self.assertCorrectItemSelected(item)

    def test_keynav_prev_selection_works(self):
        """Up key MUST select the previous valid item."""
        self.open_quicklist_with_mouse()

        for item in reversed(self.quicklist.selectable_items):
            self.keybinding("quicklist/keynav/prev")
            self.assertCorrectItemSelected(item)

    def test_keynav_prev_is_cyclic(self):
        """Up key MUST select the last item, when the first one is selected."""
        self.open_quicklist_with_mouse()

        mouse_item = self.quicklist.selectable_items[0]
        mouse_item.mouse_move_to()
        self.assertThat(mouse_item.selected, Eventually(Equals(True)))

        self.keybinding("quicklist/keynav/prev")
        expected_item = self.quicklist.selectable_items[-1]
        self.assertCorrectItemSelected(expected_item)

    def test_keynav_next_is_cyclic(self):
        """Down key MUST select the first item, when the last one is selected."""
        self.open_quicklist_with_mouse()

        mouse_item = self.quicklist.selectable_items[-1]
        mouse_item.mouse_move_to()
        self.assertThat(mouse_item.selected, Eventually(Equals(True)))

        self.keybinding("quicklist/keynav/next")
        expected_item = self.quicklist.selectable_items[0]
        self.assertCorrectItemSelected(expected_item)

    def test_keynav_mouse_interaction(self):
        """Tests that the interaction between key-navigation and mouse works as
        expected. See bug #911561.
        """
        self.open_quicklist_with_mouse()
        mouse_item = self.quicklist.selectable_items[-1]
        mouse_item.mouse_move_to()
        self.assertThat(mouse_item.selected, Eventually(Equals(True)))

        self.keybinding("quicklist/keynav/prev")
        sleep(.1)
        self.keybinding("quicklist/keynav/prev")

        key_item = self.quicklist.selectable_items[-3]
        self.assertCorrectItemSelected(key_item)

        # Moving the mouse horizontally doesn't change the selection
        self.mouse.move(mouse_item.x + mouse_item.width - 10, mouse_item.y + mouse_item.height / 2)
        self.assertThat(self.quicklist.selected_item, Equals(key_item))

        # Moving the mouse outside doesn't change the selection
        self.mouse.move(mouse_item.x + mouse_item.width + 50, mouse_item.y + mouse_item.height / 2)
        self.assertThat(self.quicklist.selected_item, Equals(key_item))

        # Moving the mouse to another entry, changes the selection
        mouse_item = self.quicklist.selectable_items[-2]
        mouse_item.mouse_move_to()
        self.assertCorrectItemSelected(mouse_item)

    def test_moving_mouse_during_grab_select_correct_menuitem(self):
        """Test that moving the mouse during grabbing selects the
        correct menu item. See bug #1027955.
        """
        self.open_quicklist_with_mouse()
        mouse_item = self.quicklist.selectable_items[0]
        mouse_item.mouse_move_to()
        self.assertThat(mouse_item.selected, Eventually(Equals(True)))

        # Dragging the mouse horizontally doesn't change the selection
        self.mouse.press()
        self.addCleanup(self.mouse.release)
        self.mouse.move(mouse_item.x + mouse_item.width - 10, mouse_item.y + mouse_item.height / 2)
        self.assertThat(mouse_item.selected, Eventually(Equals(True)))

        # Moving the mouse down selects the next item
        mouse_item = self.quicklist.selectable_items[1]
        mouse_item.mouse_move_to()
        self.assertThat(mouse_item.selected, Eventually(Equals(True)))



# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
# Copyright 2012 Canonical
# Author: Thomi Richards
#
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 3, as published
# by the Free Software Foundation.
#


from dbus import Interface, SessionBus

from unity.emulators import UnityIntrospectionObject
from unity.emulators.dash import DashController
from unity.emulators.hud import HudController
from unity.emulators.launcher import LauncherController
from unity.emulators.panel import PanelController
from unity.emulators.screen import Screen
from unity.emulators.shortcut_hint import ShortcutController
from unity.emulators.switcher import SwitcherController
from unity.emulators.window_manager import WindowManager


# acquire the debugging dbus object
UNITY_BUS_NAME = 'com.canonical.Unity'
DEBUG_PATH = '/com/canonical/Unity/Debug'
LOGGING_IFACE = 'com.canonical.Unity.Debug.Logging'


def get_dbus_proxy_object():
    return SessionBus().get_object(UNITY_BUS_NAME, DEBUG_PATH)


def get_dbus_logging_interface():
    return Interface(get_dbus_proxy_object(), LOGGING_IFACE)


def start_log_to_file(file_path):
    """Instruct Unity to start logging to the given file."""
    get_dbus_logging_interface().StartLogToFile(file_path)


def reset_logging():
    """Instruct Unity to stop logging to a file."""
    get_dbus_logging_interface().ResetLogging()


def set_log_severity(component, severity):
    """Instruct Unity to set a log component's severity.

    'component' is the unity logging component name.

    'severity' is the severity name (like 'DEBUG', 'INFO' etc.)

    """
    get_dbus_logging_interface().SetLogSeverity(component, severity)


def log_unity_message(severity, message):
    """Instruct unity to log a message for us.

    severity: one of ('TRACE', 'DEBUG', 'INFO', 'WARNING', 'ERROR').

    message: The message to log.

    For debugging purposes only! If you want to log a message during an autopilot
    test, use the python logging framework instead.

    """
    get_dbus_logging_interface().LogMessage(severity, message)


class Unity(UnityIntrospectionObject):

    @property
    def screen(self):
        return self.get_children_by_type(Screen)[0]

    @property
    def dash(self):
        return self.get_children_by_type(DashController)[0]

    @property
    def hud(self):
        return self.get_children_by_type(HudController)[0]

    @property
    def launcher(self):
        return self.get_children_by_type(LauncherController)[0]

    @property
    def panels(self):
        return self.get_children_by_type(PanelController)[0]

    @property
    def switcher(self):
        return self.get_children_by_type(SwitcherController)[0]

    @property
    def shortcut_hint(self):
        return self.get_children_by_type(ShortcutController)[0]

    @property
    def window_manager(self):
        return self.get_children_by_type(WindowManager)[0]



#!/usr/bin/env python3

"""
This file is part of "uwidgets" which is released under GPL.

See file LICENCE or go to http://www.gnu.org/licenses/ for full license
details.

uwidgets is a desktop widget creation and management library for Python 3.

Copyright (c) 2022 Rudra Saraswat <rs2009@ubuntu.com>.
Copyright (c) 2018 Gabriele N. Tornetta <phoenix1987@gmail.com>.

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.
You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""

from setuptools import Extension, find_packages, setup

x11 = Extension('uwidgets._x11',
    include_dirs       = ['/usr/include/cairo/'],
    libraries          = ['cairo', 'X11', 'Xinerama'],
    extra_compile_args = ['-std=c99'],
    sources            = [
        'uwidgets/x11/_x11module.c',
        'uwidgets/x11/atelier.c',
        'uwidgets/x11/base_canvas.c',
    ]
)


setup(
    name             = 'uwidgets',
    version          = '1.0.0',
    description      = 'Desktop Widget Manager for Unity, based on Blighty.',
    author           = 'Rudra Saraswat',
    author_email     = 'rs2009@ubuntu.com',
    url              = 'https://unityd.org',
    classifiers=[
        'Development Status :: 5 - Production/Stable',

        'Intended Audience :: Developers',
        'Topic :: Software Development :: Build Tools',

        'License :: OSI Approved :: GNU General Public License v3 (GPLv3)',

        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.11',
    ],
    keywords         = 'desklet widget infotainment',
    packages         = find_packages(exclude=['contrib', 'docs']),
    ext_modules      = [x11],
    install_requires = ['pycairo'],
    scripts          = ['uwidgets-runner'],
)



"""
This file is part of "blighty" "uwidgets" which is released under GPL.

See file LICENCE or go to http://www.gnu.org/licenses/ for full license
details.

uwidgets is a desktop widget creation and management library for Python 3.

Copyright (c) 2018 Gabriele N. Tornetta <phoenix1987@gmail.com>.
All rights reserved.

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.
You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""

from functools import wraps


def not_callable_from_instance(*args, **kwargs):
    raise RuntimeError(
        "Method not callable on instances of type {}".format(
            type(args[0]).__name__
        )
    )


class BrushSets:
    brush_sets = {}
    inherited = {}

    @staticmethod
    def add_brush(brush_set, method_name, method):
        if brush_set not in BrushSets.brush_sets:
            BrushSets.brush_sets[brush_set] = {}

        BrushSets.brush_sets[brush_set][method_name] = method

    @staticmethod
    def get_brush_set(brush_set):
        return BrushSets.brush_sets.get(brush_set, {})

    @staticmethod
    def inherit(klass):
        if klass.__qualname__ in BrushSets.inherited:
            return

        for e in klass.__bases__:
            try:
                BrushSets.inherit(e)
                if klass.__qualname__ not in BrushSets.brush_sets:
                    BrushSets.brush_sets[klass.__qualname__] = {}
                BrushSets.brush_sets[klass.__qualname__].update(BrushSets.brush_sets[e.__qualname__])
            except KeyError:
                # No brushes registered for the superclass so we can skip it
                pass

        BrushSets.inherited[klass.__qualname__] = True


def brush(f):
    """Brush decorator.

    Used to mark a bound method of a subclass of the `Canvas` class as a
    _brush_. The method is then rebound to to the extended Cairo context that
    is passed to the `on_draw` callback. The first argument should then be
    called `ctx` or `cr` instead of `self`, but this is not enforced so that
    any keyword can be chosen.

    Example:
        class BrushExample(uwidgets.x11.Canvas):
            @brush
            def brush_method(ctx, data):
                ctx.save()
                # Draw something on the Cairo context
                ctx.restore()

            def on_draw(self, ctx):
                ctx.brush_method(42)

    In the above example, the `brush_method` has been decorated with the
    `brush` decorator. Therefore it will be callable as a method of `ctx`
    rather than `self`. An attempt to call it from `self` will cause a
    `RuntimeError` since the method is now bound to `ctx`.

    The use of the `brush` decorator is not restricted to X11 canvases.
    """
    BrushSets.add_brush(*f.__qualname__.rsplit('.', 1), method = f)

    @wraps(f)
    def wrapper(*args, **kwargs):
        return not_callable_from_instance(*args, **kwargs)

    return wrapper


## Basic brushes ##############################################################

def draw_grid(ctx, x = 50, y = 50):
    w, h = ctx.canvas.get_size()

    ctx.save()

    ctx.set_source_rgba(.8, .8, .8, .8)
    ctx.set_line_width(1)
    ctx.set_font_size(9)
    for i in range(x, w, x):
        ctx.write_text(i, 0, str(i), align = TextAlign.BOTTOM_MIDDLE)
        ctx.move_to(i, 8)
        ctx.line_to(i, h)
        ctx.stroke()

    for i in range(y, h, y):
        ctx.write_text(0, i, str(i), align = TextAlign.CENTER_LEFT)
        ctx.move_to(8, i)
        ctx.line_to(w, i)
        ctx.stroke()

    ctx.restore()


class TextAlign:
    TOP_RIGHT = 1
    TOP_MIDDLE = 2
    TOP_LEFT = 3
    CENTER_RIGHT = 4
    CENTER_MIDDLE = 5
    CENTER_LEFT = 6
    BOTTOM_RIGHT = 7
    BOTTOM_MIDDLE = 8
    BOTTOM_LEFT = 9


def write_text(cr, x, y, text, align = TextAlign.TOP_LEFT):
    ex = cr.text_extents(text)

    if align <= TextAlign.TOP_LEFT:
        dy = 0
    elif align <= TextAlign.CENTER_LEFT:
        dy = ex.height // 2
    else:
        dy = ex.height

    if align % 3 == 1:
        dx = ex.width
    elif align % 3 == 2:
        dx = ex.width // 2
    else:
        dx = 0

    cr.move_to(x - dx, y + dy)
    cr.show_text(text)
    cr.stroke()

    return ex



# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
# Copyright 2013 Canonical
# Authors: Jacob Edwards
#
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 3, as published
# by the Free Software Foundation.

from autopilot.matchers import Eventually
from testtools.matchers import Equals, NotEquals
from time import sleep

from unity.tests.launcher import LauncherTestCase, _make_scenarios

class LauncherTooltipTests(LauncherTestCase):
    """Tests whether tooltips display only at appropriate times."""

    def setUp(self):
        super(LauncherTooltipTests, self).setUp()
        self.set_unity_option('launcher_hide_mode', 0)
        self.launcher_instance.move_mouse_beside_launcher()
        self.icons = self.unity.launcher.model.get_launcher_icons(visible_only=True)

    def test_launcher_tooltip_show(self):
        """Tests whether icon tooltips delay showing themselves and,
        once shown, whether subsequent icons show them instantly."""
        for i in self.icons:
            tooltip = i.get_tooltip()
            if not tooltip:
                continue
            self.assertThat(tooltip.active, Eventually(Equals(False)))

        # only reveal tooltips after short wait
        self.assertEqual(self.get_reveal_behavior(self.icons[0]), self.DELAYED)

        # must avoid the accordion effect, as the icons start to pass to quickly
        size = len(self.icons)
        if size > 5:
            size = 5

        # subsequent tooltips reveal instantly, but are destroyed on exit, meaning None
        a, b = 0, 1
        while b < size:
            self.mouse.move(self.icons[b].center.x, self.icons[b].center.y)
            self.assertThat(lambda: self.icons[b].get_tooltip(), Eventually(NotEquals(None)))
            self.assertThat(self.icons[b].get_tooltip().active, Eventually(Equals(True)))
            self.assertThat(self.icons[a].get_tooltip(), Equals(None))
            a, b = a + 1, b + 1
        b -= 1

        # leaving launcher clears tooltips, and instant reveal
        self.launcher_instance.move_mouse_beside_launcher()
        self.assertEqual(self.get_reveal_behavior(self.icons[b]), self.DELAYED)

    def test_launcher_tooltip_disabling(self):
        """Tests whether clicking on an icon hides its tooltip."""
        bfb, other = self.icons[0], self.icons[1]
        self.assertEqual(self.get_reveal_behavior(bfb), self.DELAYED)

        # clicking icon hides its launcher until further input
        self.mouse.click()
        self.assertEqual(self.get_reveal_behavior(bfb), self.NEVER)
        self.mouse.click()

        # normal behavior resumes on moving away from icon
        self.assertEqual(self.get_reveal_behavior(other), self.DELAYED)
        self.assertEqual(self.get_reveal_behavior(bfb), self.INSTANT)

    def test_launcher_bfb_tooltip_when_open(self):
        """Tests whether hovering over the active BFB starts a tooltip timer"""
        self.unity.dash.ensure_visible()
        self.addCleanup(self.unity.dash.ensure_hidden)
        bfb, other = self.icons[0], self.icons[1]

        # hovering an open dash's BFB does not show a tooltip ...
        self.assertEqual(self.get_reveal_behavior(bfb), self.NEVER)

        # ... nor did it timeout instant tooltips for other icons
        self.assertEqual(self.get_reveal_behavior(other), self.INSTANT)

    # Tooltip reveal types
    (INSTANT, DELAYED, NEVER) = range(3)

    def get_reveal_behavior(self, icon):
        self.mouse.move(icon.center.x, icon.center.y)
        tooltip = icon.get_tooltip()
        if tooltip and tooltip.active:
            return self.INSTANT
        sleep(1.2)
        tooltip = icon.get_tooltip()
        if tooltip and tooltip.active:
            return self.DELAYED
        return self.NEVER



import os
from gi.repository import Gio, GLib


class UnityLauncher:
    launcher_settings = Gio.Settings.new('com.canonical.Unity.Launcher')
    launcher_compiz_settings = Gio.Settings.new_with_path('org.compiz.unityshell', '/org/compiz/profiles/unity/plugins/unityshell/')

    def set_launcher_position(self, position: str):
        if position == 'Bottom' or position == 'bottom':
            self.launcher_settings['launcher-position'] = 'Bottom'
        elif position == 'Left' or position == 'left':
            self.launcher_settings['launcher-position'] = 'Left'

    def set_launcher_autohide(self, hidemode: bool):
        self.launcher_compiz_settings['launcher-hide-mode'] = hidemode

# XXX: Add more settings



import os
from gi.repository import Gio, GLib

import requests


class UnityWallpaper:
    cache_dir = GLib.get_user_cache_dir()
    pictures_dir = GLib.get_user_special_dir(GLib.USER_DIRECTORY_PICTURES)
    splash_wallpaper_file_path = os.path.join(
        cache_dir,
        "org.unityd.wallpapers"
    )
    background_settings = Gio.Settings.new("org.gnome.desktop.background")

    def set_wallpaper_from_file_uri(self, file_uri: str):
        self.background_settings['picture-uri'] = f"file:///{file_uri}"

    def write_image_url_to_wallpaper_file(self, image_url: str):
        response = requests.get(image_url, stream=True)
        if response.ok:
            with open(self.splash_wallpaper_file_path, "wb") as wallpaper_path:
                wallpaper_path.write(response.raw.read())

    def set_wallpaper_from_url(self, image_url: str):
        self.write_image_url_to_wallpaper_file(image_url)
        self.set_wallpaper_from_file_uri(self.splash_wallpaper_file_path)



# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
# Copyright 2012 Canonical
# Author: Marco Trevisan (Treviño)
#
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 3, as published
# by the Free Software Foundation.
#


import logging
from autopilot.introspection.types import Rectangle
from autopilot.keybindings import KeybindingsHelper

from unity.emulators import UnityIntrospectionObject

logger = logging.getLogger(__name__)


class WindowManager(UnityIntrospectionObject, KeybindingsHelper):
    """The WindowManager class."""

    @property
    def screen_geometry(self):
        """Returns a Rectangle (x,y,w,h) for the screen."""
        return self.globalRect

    def initiate_spread(self):
        self.keybinding("spread/start")
        self.scale_active.wait_for(True)

    def terminate_spread(self):
        self.keybinding("spread/cancel")
        self.scale_active.wait_for(False)

    def enter_show_desktop(self):
        if not self.showdesktop_active:
            logger.info("Entering show desktop mode.")
            self.keybinding("window/show_desktop")
            self.showdesktop_active.wait_for(True)
        else:
            logger.warning("Test tried to enter show desktop mode while already \
                in show desktop mode.")

    def leave_show_desktop(self):
        if self.showdesktop_active:
            logger.info("Leaving show desktop mode.")
            self.keybinding("window/show_desktop")
            self.showdesktop_active.wait_for(False)
        else:
            logger.warning("Test tried to leave show desktop mode while not in \
                show desktop mode.")



#!/usr/bin/env python3

from math import pi as PI

import configparser
import psutil
import cairo
import os
from uwidgets import CanvasGravity, TextAlign
from uwidgets.legacy import Graph
from uwidgets.x11 import Canvas, start_event_loop


class AttrDict(dict):
    def __init__(self, *args, **kwargs):
        super(AttrDict, self).__init__(*args, **kwargs)
        self.__dict__ = self

class Fonts:
    UBUNTU_NORMAL = "Ubuntu", cairo.FontSlant.NORMAL, cairo.FontWeight.NORMAL


class Cpu(Canvas):
    SIZE = (256, 256)
    CORE_POLYGON = AttrDict({"height": 30, "length": 20})

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

        with open('/proc/cpuinfo', 'r') as fin:
            raw_cpuinfo = fin.read().strip()

        self.coreinfo = [
            {
                k.strip(): v
                for k, v in [p.split(":") for p in core.split('\n')]
            }
            for core in raw_cpuinfo.split('\n\n')
        ]

        self.graph = Graph(0, 110, self.width, 40)

    @staticmethod
    def build(x = 0, y = 0, gravity = CanvasGravity.CENTER):
        return Cpu(x, y, *Cpu.SIZE, gravity = gravity, interval = 2000)

    def on_button_pressed(self, button, *args):
        os.system('stacer')

    def draw_polygon(c, n, x, y, size):
        a = 2 * PI / n

        c.save()

        c.translate(x, y)
        c.move_to(size, 0)
        for i in range(n):
            c.rotate(a)
            c.line_to(size, 0)
        c.stroke()

        c.restore()

    def draw_core_polygon(c, x, y):
        size = Cpu.CORE_POLYGON.height
        length = Cpu.CORE_POLYGON.length

        c.save()

        c.translate(x, y)

        c.set_source_rgb(.8, .8, .8)

        cpus = psutil.cpu_percent(0.1, percpu = True)
        n = len(cpus)
        a = 2 * PI / n

        c.set_line_width(1)
        c.draw_polygon(n, 0, 0, size)

        c.set_line_width(2)
        c.set_source_rgb(1, 1, 1)
        c.move_to(size + length * cpus[-1] / 100, 0)
        for i in range(n):
            c.rotate(a)
            c.line_to(size + length * cpus[i] / 100, 0)
        c.stroke()

        value = int(sum(cpus) / n)
        c.canvas.graph.push_value(value)

        c.set_font_size(18)
        c.write_text(0, 0, '{}%'.format(value), TextAlign.CENTER_MIDDLE)

        c.restore()

        return value

    def draw_processes(c):
        ps = [
            p.info
            for p in psutil.process_iter(attrs=['pid', 'name', 'cpu_percent'])
        ]

        ps = sorted(ps, key=lambda p: p["cpu_percent"], reverse=True)[:5]

        y = 170
        c.save()
        c.select_font_face(*Fonts.UBUNTU_NORMAL)
        c.set_font_size(12)
        for p in ps:
            c.write_text(48, y, str(p["pid"]), align = TextAlign.TOP_RIGHT)
            c.write_text(52, y, p["name"][:24])
            c.write_text(
                c.canvas.width - 15, y, "{}%".format(p["cpu_percent"]),
                align=TextAlign.TOP_RIGHT
            )
            y += 18

        c.restore()

    def draw_cpu_name(c):
        c.save()
        c.set_font_size(12)
        c.write_text(
            15, 110,
            c.canvas.coreinfo[0]["model name"].strip()
            .replace("(TM)", "™")
            .replace("(R)", "©")
        )
        c.restore()

    def draw_background(c):
        size = c.canvas.get_size()
        c.rectangle(0, 0, *size)
        c.set_source_rgba(0, 0, 0, 0.6)
        c.fill()

        c.rectangle(0, 0, c.canvas.width, 3)
        c.set_source_rgba(1, 1, 1, 1)
        c.fill()

    def on_draw(self, c):
        c.draw_background()

        c.select_font_face(*Fonts.UBUNTU_NORMAL)
        c.set_font_size(36)
        c.set_source_rgb(1, 1, 1)

        w, h = Cpu.SIZE

        y_poly = (Cpu.CORE_POLYGON.height + Cpu.CORE_POLYGON.length)

        c.write_text(15, y_poly, "CPU", align = TextAlign.TOP_LEFT)
        c.draw_core_polygon(w - y_poly, y_poly)
        c.draw_processes()
        c.draw_cpu_name()

        c.set_source_rgb(1, 1, 1)
        self.graph.draw(c)


if __name__ == "__main__":
    config = configparser.ConfigParser()
    config.read('settings.ini')
    Cpu.build(int(config.get('settings', 'margin_x')),
              int(config.get('settings', 'margin_y')),
              gravity = getattr(CanvasGravity, config.get('settings', 'gravity'))).show()
    start_event_loop()



import apport.packaging
from apport.hookutils import *

def add_info(report, ui):

    # for install from the ppa
    if not apport.packaging.is_distro_package(report['Package'].split()[0]):
        report['CrashDB'] = 'unity'
        try:
            version = packaging.get_version('unity')
        except ValueError:
            version = 'N/A'
        if version is None:
            version = 'N/A'
        report['Tags'] += " rc-%s" % version
    
    # the crash is not in the unity code so reassign
    if "Stacktrace" in report and "/usr/lib/indicators" in report["Stacktrace"]:
        for words in report["Stacktrace"].split():
            if words.startswith("/usr/lib/indicators"):
                report.add_package_info(apport.packaging.get_file_package(words))
                return

    # Include the compiz details
    report.add_hooks_info(ui, srcpackage='compiz')
    # the upstart logs
    attach_upstart_logs(report, 'unity-services')
    attach_upstart_logs(report ,'libunity-core-6.0-9')
    # some gsettings configs
    attach_gsettings_schema(report, 'com.canonical.Unity')
    attach_gsettings_schema(report, 'com.ubuntu.user-interface')
    attach_gsettings_schema(report, 'org.gnome.desktop.interface')
    attach_gsettings_schema(report, 'org.gnome.desktop.lockdown')



# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
# Copyright 2012 Canonical
# Author: Brandon Schaefer
#
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 3, as published
# by the Free Software Foundation.


from autopilot.matchers import Eventually
from os import environ
import subprocess
from testtools.matchers import Equals
from testtools import skip
from unity.tests import UnityTestCase


class GcinTestCase(UnityTestCase):
    """Tests the Input Method gcin."""

    def setUp(self):
        super(GcinTestCase, self).setUp()

        # Check that gcin is set as the active IM through im-switch
        if environ.get('XMODIFIERS', '') != "@im=gcin":
            self.skip("Please make sure XMODIFIERS is set to @im=gcin. Set it using 'im-switch'.")

        running_process = subprocess.check_output('ps -e', shell=True)
        if 'gcin' not in running_process:
            self.skip("gcin is not an active process, please start 'gcin' before running these tests.")

class GcinTestHangul(GcinTestCase):
    """Tests the Dash and Hud with gcin in hangul mode."""

    scenarios = [
            ('hangul', {'input': 'han geul ', 'result': '\ud55c\uae00'}),
            ('morning letters', {'input': 'a chimgeul ', 'result': '\uc544\uce68\uae00'}),
            ('national script', {'input': 'gug mun ', 'result': '\uad6d\ubb38'}),
        ]

    def setUp(self):
        super(GcinTestHangul, self).setUp()

    def enter_hangul_mode(self):
        """Ctrl+Space turns gcin on, Ctrl+Alt+/ turns hangul on."""
        self.keyboard.press_and_release("Ctrl+Space")
        self.keyboard.press_and_release("Ctrl+Alt+/")

    def test_dash_input(self):
        """Entering an input string through gcin will result in a Korean string result in the dash."""

        self.unity.dash.ensure_visible()
        self.addCleanup(self.unity.dash.ensure_hidden)
        self.enter_hangul_mode()

        self.keyboard.type(self.input)
        self.assertThat(self.unity.dash.search_string, Eventually(Equals(self.result)))

    def test_hud_input(self):
        """Entering an input string through gcin will result in a Korean string result in the hud."""

        self.unity.hud.ensure_visible()
        self.addCleanup(self.unity.hud.ensure_hidden)
        self.enter_hangul_mode()

        self.keyboard.type(self.input)
        self.assertThat(self.unity.hud.search_string, Eventually(Equals(self.result)))



# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
# Copyright 2012 Canonical
# Authors: Thomi Richards,
#          Marco Trevisan (Treviño)
#
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 3, as published
# by the Free Software Foundation.


from autopilot.matchers import Eventually
from autopilot.testcase import multiply_scenarios
import logging
from testtools.matchers import Equals, NotEquals, GreaterThan
from time import sleep

from unity.emulators.icons import ApplicationLauncherIcon, ExpoLauncherIcon
from unity.emulators.launcher import IconDragType
from unity.emulators.launcher import LauncherPosition
from unity.tests.launcher import LauncherTestCase, _make_scenarios

from Xlib import Xutil

logger = logging.getLogger(__name__)


class LauncherIconsTests(LauncherTestCase):
    """Test the launcher icons interactions"""

    def setUp(self):
        super(LauncherIconsTests, self).setUp()

        self.set_unity_option('launcher_hide_mode', 0)

    def ensure_expo_launcher_icon(self):
        EXPO_URI = "'unity://expo-icon'"
        old_fav = self.call_gsettings_cmd('get', 'com.canonical.Unity.Launcher', 'favorites')

        if not EXPO_URI in old_fav:
            if old_fav[:-2] == "[]":
                new_fav = "["+EXPO_URI+"]"
            else:
                new_fav = old_fav[:-1]+", "+EXPO_URI+"]"

            self.addCleanup(self.call_gsettings_cmd, 'set', 'com.canonical.Unity.Launcher', 'favorites', old_fav)
            self.call_gsettings_cmd('set', 'com.canonical.Unity.Launcher', 'favorites', new_fav)

        icon = self.unity.launcher.model.get_children_by_type(ExpoLauncherIcon)[0]
        self.assertThat(icon, NotEquals(None))
        self.assertThat(icon.visible, Eventually(Equals(True)))

        return icon

    def ensure_calculator_in_launcher_and_not_running(self):
        calc = self.process_manager.start_app("Calculator")
        calc_icon = self.unity.launcher.model.get_icon(desktop_id=calc.desktop_file)
        self.addCleanup(self.launcher_instance.unlock_from_launcher, calc_icon)
        self.launcher_instance.lock_to_launcher(calc_icon)
        self.process_manager.close_all_app("Calculator")
        self.assertThat(lambda: self.process_manager.app_is_running("Calculator"), Eventually(Equals(False)))
        return calc_icon

    def get_running_application_by_desktop_file(self, desktop_id):
        get_app_fn = lambda: self.process_manager.get_running_applications_by_desktop_file(desktop_id)
        self.assertThat(lambda: len(get_app_fn()), Eventually(Equals(1)))
        return get_app_fn()[0]

    def test_bfb_tooltip_disappear_when_dash_is_opened(self):
        """Tests that the bfb tooltip disappear when the dash is opened."""
        bfb = self.unity.launcher.model.get_bfb_icon()
        self.mouse.move(bfb.center.x, bfb.center.y)

        self.assertThat(lambda: bfb.get_tooltip(), Eventually(NotEquals(None)))
        self.assertThat(bfb.get_tooltip().active, Eventually(Equals(True)))
        self.unity.dash.ensure_visible()
        self.addCleanup(self.unity.dash.ensure_hidden)

        self.assertThat(bfb.get_tooltip(), Equals(None))

    def test_bfb_tooltip_is_disabled_when_dash_is_open(self):
        """Tests the that bfb tooltip is disabled when the dash is open."""
        self.unity.dash.ensure_visible()
        self.addCleanup(self.unity.dash.ensure_hidden)

        bfb = self.unity.launcher.model.get_bfb_icon()
        self.mouse.move(bfb.center.x, bfb.center.y)

        # Tooltips are lazy-created  in Unity, so if the BFB tooltip has never
        # been shown before, get_tooltip will return None. If that happens, then
        # this test should pass.
        tooltip = bfb.get_tooltip()
        if tooltip is not None:
            self.assertThat(tooltip.active, Eventually(Equals(False)))

    def test_shift_click_opens_new_application_instance(self):
        """Shift+Clicking MUST open a new instance of an already-running application."""
        app = self.process_manager.start_app("Calculator")
        icon = self.unity.launcher.model.get_icon(desktop_id=app.desktop_file)

        self.keyboard.press("Shift")
        self.addCleanup(self.keyboard.release, "Shift")
        self.launcher_instance.click_launcher_icon(icon)

        self.assertNumberWinsIsEventually(app, 2)

    def test_launcher_activate_last_focused_window(self):
        """Activating a launcher icon must raise only the last focused instance
        of that application.

        """
        char_win1 = self.process_manager.start_app_window("Character Map")
        calc_win = self.process_manager.start_app_window("Calculator")
        char_win2 = self.process_manager.start_app_window("Character Map")

        self.assertVisibleWindowStack([char_win2, calc_win, char_win1])

        char_icon = self.unity.launcher.model.get_icon(
            desktop_id=char_win2.application.desktop_file)
        calc_icon = self.unity.launcher.model.get_icon(
            desktop_id=calc_win.application.desktop_file)

        self.launcher_instance.click_launcher_icon(calc_icon)
        self.assertProperty(calc_win, is_focused=True)
        self.assertVisibleWindowStack([calc_win, char_win2, char_win1])

        self.launcher_instance.click_launcher_icon(char_icon)
        self.assertProperty(char_win2, is_focused=True)
        self.assertVisibleWindowStack([char_win2, calc_win, char_win1])

        self.keybinding("window/minimize")

        self.assertThat(lambda: char_win2.is_hidden, Eventually(Equals(True)))
        self.assertProperty(calc_win, is_focused=True)
        self.assertVisibleWindowStack([calc_win, char_win1])

        self.launcher_instance.click_launcher_icon(char_icon)
        self.assertProperty(char_win1, is_focused=True)
        self.assertThat(lambda: char_win2.is_hidden, Eventually(Equals(True)))
        self.assertVisibleWindowStack([char_win1, calc_win])

    def test_launcher_uses_startup_notification(self):
        """Tests that unity uses startup notification protocol."""
        calc_icon = self.ensure_calculator_in_launcher_and_not_running()
        self.addCleanup(self.process_manager.close_all_app, "Calculator")
        self.launcher_instance.click_launcher_icon(calc_icon)

        calc_app = self.get_running_application_by_desktop_file(calc_icon.desktop_id)
        calc_window = calc_app.get_windows()[0]

        self.assertThat(lambda: self.get_startup_notification_timestamp(calc_window), Eventually(Equals(calc_icon.startup_notification_timestamp)))

    def test_trash_icon_refocus_opened_instance(self):
        """Tests that when the trash is opened, clicking on the icon re-focus the trash again"""
        self.register_nautilus()
        self.addCleanup(self.close_all_windows, "Nautilus")
        self.addCleanup(self.process_manager.close_all_app, "Calculator")
        self.close_all_windows("Nautilus")

        trash_icon = self.unity.launcher.model.get_trash_icon()
        self.launcher_instance.click_launcher_icon(trash_icon)
        self.assertThat(lambda: len(self.process_manager.get_open_windows_by_application("Nautilus")), Eventually(Equals(1)))
        [trash_window] = self.process_manager.get_open_windows_by_application("Nautilus")
        self.assertThat(lambda: trash_window.is_focused, Eventually(Equals(True)))

        calc_win = self.process_manager.start_app_window("Calculator")
        self.assertThat(lambda: calc_win.is_focused, Eventually(Equals(True)))
        self.assertThat(lambda: trash_window.is_focused, Eventually(Equals(False)))

        self.launcher_instance.click_launcher_icon(trash_icon)
        self.assertThat(lambda: trash_window.is_focused, Eventually(Equals(True)))

    def test_trash_open_does_not_prevent_nautilus_to_run(self):
        """Tests that when the trash is opened, launching still opens a new window"""
        self.register_nautilus()
        self.addCleanup(self.close_all_windows, "Nautilus")
        self.close_all_windows("Nautilus")

        trash_icon = self.unity.launcher.model.get_trash_icon()
        self.launcher_instance.click_launcher_icon(trash_icon)
        self.assertThat(lambda: len(self.process_manager.get_open_windows_by_application("Nautilus")), Eventually(Equals(1)))

        nautilus_app = self.process_manager.get_app_instances("Nautilus")
        nautilus_icon = self.unity.launcher.model.get_icon(desktop_id="org.gnome.Nautilus.desktop")
        self.launcher_instance.click_launcher_icon(nautilus_icon)
        self.assertThat(lambda: len(self.process_manager.get_open_windows_by_application("Nautilus")), Eventually(Equals(2)))

    def test_super_number_shortcut_focuses_new_windows(self):
        """Windows launched using super+number must have
        keyboard focus.

        """
        bfb_icon = self.unity.launcher.model.get_bfb_icon()
        calc_icon = self.ensure_calculator_in_launcher_and_not_running()
        self.addCleanup(self.process_manager.close_all_app, "Calculator")

        self.launcher_instance.drag_icon_to_position(
            calc_icon,
            IconDragType.AFTER,
            bfb_icon,
            launcher_position = self.launcher_position)

        self.launcher_instance.keyboard_reveal_launcher()
        self.addCleanup(self.launcher_instance.keyboard_unreveal_launcher)
        self.keyboard.press_and_release("1");

        calc_app = self.get_running_application_by_desktop_file(calc_icon.desktop_id)
        [calc_window] = calc_app.get_windows()

        self.assertThat(lambda: calc_window.is_focused, Eventually(Equals(True)))

    def test_clicking_icon_twice_initiates_spread(self):
        """This tests shows that when you click on a launcher icon twice,
        when an application window is focused, the spread is initiated.
        """
        char_win1 = self.process_manager.start_app_window("Character Map")
        char_win2 = self.process_manager.start_app_window("Character Map")
        char_app = char_win1.application

        self.assertVisibleWindowStack([char_win2, char_win1])
        self.assertProperty(char_win2, is_focused=True)

        char_icon = self.unity.launcher.model.get_icon(desktop_id=char_app.desktop_file)
        self.addCleanup(self.keybinding, "spread/cancel")
        self.launcher_instance.click_launcher_icon(char_icon)

        self.assertThat(self.unity.window_manager.scale_active, Eventually(Equals(True)))
        self.assertThat(self.unity.window_manager.scale_active_for_group, Eventually(Equals(True)))

    def test_while_in_scale_mode_the_dash_will_still_open(self):
        """If scale is initiated through the laucher pressing super must close
        scale and open the dash.
        """
        char_win1 = self.process_manager.start_app_window("Character Map")
        char_win2 = self.process_manager.start_app_window("Character Map")
        char_app = char_win1.application

        self.assertVisibleWindowStack([char_win2, char_win1])
        self.assertProperty(char_win2, is_focused=True)

        char_icon = self.unity.launcher.model.get_icon(desktop_id=char_app.desktop_file)
        self.launcher_instance.click_launcher_icon(char_icon)
        self.assertThat(self.unity.window_manager.scale_active, Eventually(Equals(True)))

        self.unity.dash.ensure_visible()
        self.addCleanup(self.unity.dash.ensure_hidden)

        self.assertThat(self.unity.dash.visible, Eventually(Equals(True)))
        self.assertThat(self.unity.window_manager.scale_active, Eventually(Equals(False)))

    def test_right_click_on_icon_ends_expo(self):
        """Right click on a launcher icon in expo mode must end the expo
        and show the quicklist.

        """
        if self.workspace.num_workspaces <= 1:
            self.skipTest("This test requires enabled workspaces.")

        self.keybinding("expo/start")
        self.assertThat(self.unity.window_manager.expo_active, Eventually(Equals(True)))
        self.addCleanup(self.keybinding, "expo/cancel")

        bfb = self.unity.launcher.model.get_bfb_icon()
        self.mouse.move(bfb.center.x, bfb.center.y)
        self.mouse.click(button=3)

        self.assertThat(self.launcher_instance.quicklist_open, Eventually(Equals(True)))
        self.assertThat(self.unity.window_manager.expo_active, Eventually(Equals(False)))

    def test_expo_launcher_icon_initiates_expo(self):
        """Clicking on the expo launcher icon must start the expo."""
        if self.workspace.num_workspaces <= 1:
            self.skipTest("This test requires enabled workspaces.")

        expo = self.ensure_expo_launcher_icon()
        self.addCleanup(self.keybinding, "expo/cancel")
        self.launcher_instance.click_launcher_icon(expo)

        self.assertThat(self.unity.window_manager.expo_active, Eventually(Equals(True)))

    def test_expo_launcher_icon_terminates_expo(self):
        """Clicking on the expo launcher icon when expo is active must close it."""
        if self.workspace.num_workspaces <= 1:
            self.skipTest("This test requires enabled workspaces.")

        self.keybinding("expo/start")
        self.assertThat(self.unity.window_manager.expo_active, Eventually(Equals(True)))
        self.addCleanup(self.keybinding, "expo/cancel")

        expo = self.ensure_expo_launcher_icon()
        self.launcher_instance.click_launcher_icon(expo)

        self.assertThat(self.unity.window_manager.expo_active, Eventually(Equals(False)))

    def test_unminimize_initially_minimized_windows(self):
        """Make sure initially minimized windows can be unminimized."""
        window_spec = {
            "Title": "Hello",
            "Minimized": True
        }

        window = self.launch_test_window(window_spec)
        icon = self.unity.launcher.model.get_icon(desktop_id=window.application.desktop_file)

        self.launcher_instance.click_launcher_icon(icon)
        self.assertThat(lambda: window.x_win.get_wm_state()['state'], Eventually(Equals(Xutil.NormalState)))

    def test_unminimize_minimized_immediately_after_show_windows(self):
        """Make sure minimized-immediately-after-show windows can be unminimized."""
        window_spec = {
            "Title": "Hello",
            "MinimizeImmediatelyAfterShow": True
        }

        window = self.launch_test_window(window_spec)
        icon = self.unity.launcher.model.get_icon(desktop_id=window.application.desktop_file)

        self.launcher_instance.click_launcher_icon(icon)
        self.assertThat(lambda: window.x_win.get_wm_state()['state'], Eventually(Equals(Xutil.NormalState)))

    def test_icon_focuses_application_window_when_dash_is_open(self):
        """Clicking on an icon when dash is shown must focus it."""
        win = self.process_manager.start_app_window("Calculator")
        self.assertThat(lambda: win.is_focused, Eventually(Equals(True)))

        self.addCleanup(self.unity.dash.ensure_hidden)
        self.unity.dash.ensure_visible()
        self.assertThat(lambda: win.is_focused, Eventually(Equals(False)))

        icon = self.unity.launcher.model.get_icon(desktop_id=win.application.desktop_file)
        self.launcher_instance.click_launcher_icon(icon)

        self.assertThat(self.unity.dash.visible, Eventually(Equals(False)))
        self.assertThat(lambda: win.is_focused, Eventually(Equals(True)))

    def test_icon_focuses_application_window_when_hud_is_open(self):
        """Clicking on an icon when hud is shown must focus it."""
        win = self.process_manager.start_app_window("Calculator")
        self.assertThat(lambda: win.is_focused, Eventually(Equals(True)))

        self.addCleanup(self.unity.hud.ensure_hidden)
        self.unity.hud.ensure_visible()
        self.assertThat(lambda: win.is_focused, Eventually(Equals(False)))

        icon = self.unity.launcher.model.get_icon(desktop_id=win.application.desktop_file)
        self.launcher_instance.click_launcher_icon(icon)

        self.assertThat(self.unity.hud.visible, Eventually(Equals(False)))
        self.assertThat(lambda: win.is_focused, Eventually(Equals(True)))


class LauncherDragIconsBehavior(LauncherTestCase):
    """Tests dragging icons around the Launcher."""

    scenarios = multiply_scenarios(_make_scenarios(),
                                   [
                                       ('inside', {'drag_type': IconDragType.INSIDE}),
                                       ('outside', {'drag_type': IconDragType.OUTSIDE}),
                                   ],
                                   [
                                       ('left', {'launcher_position': LauncherPosition.LEFT}),
                                       ('bottom', {'launcher_position': LauncherPosition.BOTTOM}),
                                   ])

    def setUp(self):
        super(LauncherDragIconsBehavior, self).setUp()

        self.set_unity_option('launcher_hide_mode', 0)

    def ensure_calc_icon_not_in_launcher(self):
        """Wait until the launcher model updates and removes the calc icon."""
        # Normally we'd use get_icon(desktop_id="...") but we're expecting it to
        # not exist, and we don't want to wait for 10 seconds, so we do this
        # the old fashioned way.
        get_icon_fn = lambda: self.unity.launcher.model.get_children_by_type(
            ApplicationLauncherIcon, desktop_id="gcalctool.desktop")
        calc_icon = get_icon_fn()
        if calc_icon:
            self.launcher_instance.unlock_from_launcher(calc_icon[0])

        self.assertThat(get_icon_fn, Eventually(Equals([])))

    def test_can_drag_icon_below_bfb(self):
        """Application icons must be draggable to below the BFB."""

        self.ensure_calc_icon_not_in_launcher()
        calc = self.process_manager.start_app("Calculator")
        calc_icon = self.unity.launcher.model.get_icon(desktop_id=calc.desktop_file)
        bfb_icon = self.unity.launcher.model.get_bfb_icon()

        self.launcher_instance.drag_icon_to_position(
            calc_icon,
            IconDragType.AFTER,
            bfb_icon,
            self.drag_type,
            self.launcher_position)
        moved_icon = self.unity.launcher.model.\
                     get_launcher_icons_for_monitor(self.launcher_monitor)[1]
        self.assertThat(moved_icon, Equals(calc_icon))

    def test_can_drag_icon_below_window_switcher(self):
        """Application icons must be dragable to below the workspace switcher icon."""

        self.ensure_calc_icon_not_in_launcher()
        calc = self.process_manager.start_app("Calculator")
        calc_icon = self.unity.launcher.model.get_icon(desktop_id=calc.desktop_file)
        bfb_icon = self.unity.launcher.model.get_bfb_icon()
        trash_icon = self.unity.launcher.model.get_trash_icon()

        # Move a known icon to the top as it needs to be more than 2 icon
        # spaces away for this test to actually do anything
        self.launcher_instance.drag_icon_to_position(
            calc_icon,
            IconDragType.AFTER,
            bfb_icon,
            self.drag_type,
            self.launcher_position)

        sleep(1)
        self.launcher_instance.drag_icon_to_position(
            calc_icon,
            IconDragType.BEFORE,
            trash_icon,
            self.drag_type,
            self.launcher_position)

        # Must be the last bamf icon - not necessarily the third-from-end icon.
        expected_pos = -2 if self.workspace.num_workspaces < 2 else -1
        refresh_fn = lambda: self.unity.launcher.model.get_launcher_icons()[expected_pos].id
        self.assertThat(refresh_fn,
            Eventually(Equals(calc_icon.id)),
            "Launcher icons are: %r" % self.unity.launcher.model.get_launcher_icons())



#!/usr/bin/env python3

"""
This file is part of "blighty" and "uwidgets" which is released under GPL.

See file LICENCE or go to http://www.gnu.org/licenses/ for full license
details.

uwidgets is a desktop widget creation and management library for Python 3.

Copyright (c) 2022 Rudra Saraswat <rs2009@ubuntu.com>.
Copyright (c) 2018 Gabriele N. Tornetta <phoenix1987@gmail.com>.
All rights reserved.

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.
You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""

from uwidgets import CanvasGravity, brush
from uwidgets.x11 import Canvas, start_event_loop

import datetime
import subprocess
import configparser

from math import pi as PI


class Clock(Canvas):
    def on_button_pressed(self, button, state, x, y):
        if button == 3:    # Right button
            subprocess.run(['unity-control-center', 'datetime'])

    def draw_circle_background(ctx):
        ctx.arc(2, 1, 90, 0, 2*PI)
        ctx.set_source_rgba(0, 0, 0, 0.6)
        ctx.fill()

    def draw_rect_background(ctx):
        ctx.rectangle(-180, -180, 360, 360)
        ctx.set_source_rgba(0, 0, 0, 0.6)
        ctx.fill()

        ctx.rectangle(-180, -100, 360, 3)
        ctx.set_source_rgba(1, 1, 1, 1)
        ctx.fill()

    @brush
    def hand(ctx, angle, length, thickness):
        ctx.save()
        ctx.set_source_rgba(1, 1, 1, 1)
        ctx.set_line_width(thickness)
        ctx.rotate(angle)
        ctx.move_to(0, length * .2)
        ctx.line_to(0, -length)
        ctx.stroke()
        ctx.restore()

    def on_draw(self, ctx):
        now = datetime.datetime.now()

        ctx.translate(self.width >> 1, self.height >> 1)

        getattr(ctx, f"draw_{config.get('settings', 'clock_style')}_background")()

        ctx.hand(
            angle = now.second / 30 * PI,
            length = ((self.height >> 1) * .9) - 20,
            thickness = 1
        )

        mins = now.minute + now.second / 60
        ctx.hand(
            angle = mins / 30 * PI,
            length = ((self.height >> 1) * .8) - 20,
            thickness = 3
        )

        hours = (now.hour % 12) + mins / 60
        ctx.hand(
            angle = hours / 6 * PI,
            length = ((self.height >> 1) * .5) - 20,
            thickness = 4.5
        )

if __name__ == "__main__":
    config = configparser.ConfigParser()
    config.read('settings.ini')
    clock = Clock(int(config.get('settings', 'margin_x')),
                  int(config.get('settings', 'margin_y')),
                  200,
                  200,
                  gravity = getattr(CanvasGravity, str(config.get('settings', 'gravity'))))
    clock.show()
    start_event_loop()



# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
# Copyright 2012 Canonical
# Authors: Marco Trevisan (Treviño)
#
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 3, as published
# by the Free Software Foundation.


from autopilot.matchers import Eventually
from testtools.matchers import Equals
from time import sleep

from unity.emulators.shortcut_hint import ShortcutController
from unity.tests import UnityTestCase


class BaseShortcutHintTests(UnityTestCase):
    """Base class for the shortcut hint tests"""

    def setUp(self):
        super(BaseShortcutHintTests, self).setUp()

        self.DEFAULT_WIDTH = 970;
        self.DEFAULT_HEIGHT = 680;

        #self.shortcut_hint = self.get_shortcut_controller()
        self.set_unity_option('shortcut_overlay', True)
        self.set_unity_log_level("unity.shell.compiz", "DEBUG")
        self.skip_if_monitor_too_small()
        sleep(1)

    def skip_if_monitor_too_small(self):
        monitor = self.display.get_primary_screen()
        monitor_geo = self.display.get_screen_geometry(monitor)
        monitor_w = monitor_geo[2]
        monitor_h = monitor_geo[3]
        launcher_width = self.unity.launcher.get_launcher_for_monitor(monitor).geometry[2]
        panel_height = self.unity.panels.get_panel_for_monitor(monitor).geometry[3]

        if ((monitor_w - launcher_width) <= self.DEFAULT_WIDTH or
            (monitor_h - panel_height) <= self.DEFAULT_HEIGHT):
            self.skipTest("This test requires a bigger screen, to show the ShortcutHint")

    # def get_shortcut_controller(self):
    #     controllers = ShortcutController.get_all_instances()
    #     self.assertThat(len(controllers), Equals(1))
    #     return controllers[0]

    def get_launcher(self):
        # We could parameterise this so all tests run on both monitors (if MM is
        # set up), but I think it's fine to just always use monitor primary monitor:
        monitor = self.display.get_primary_screen()
        return self.unity.launcher.get_launcher_for_monitor(monitor)


class ShortcutHintTests(BaseShortcutHintTests):
    """Tests for the shortcut hint functionality in isolation."""

    def test_shortcut_hint_reveal(self):
        """Test that the shortcut hint is shown."""
        self.unity.shortcut_hint.show()
        self.addCleanup(self.unity.shortcut_hint.ensure_hidden)
        self.assertThat(self.unity.shortcut_hint.visible, Eventually(Equals(True)))

    def test_shortcut_hint_reveal_timeout(self):
        """Shortcut hint must be shown after a sufficient timeout."""
        timeout = self.unity.shortcut_hint.get_show_timeout()
        self.unity.shortcut_hint.show()
        self.addCleanup(self.unity.shortcut_hint.ensure_hidden)

        sleep(timeout/2.0)
        self.assertThat(self.unity.shortcut_hint.visible, Equals(False))
        # This should happen after 3/4 of 'timeout':
        self.assertThat(self.unity.shortcut_hint.visible, Eventually(Equals(True)))

    def test_shortcut_hint_unreveal(self):
        """Shortcut hint must hide when keys are released."""
        self.unity.shortcut_hint.ensure_visible()
        self.unity.shortcut_hint.hide()
        self.assertThat(self.unity.shortcut_hint.visible, Eventually(Equals(False)))

    def test_shortcut_hint_cancel(self):
        """Shortcut hint must hide when cancelled."""
        self.unity.shortcut_hint.ensure_visible()
        self.unity.shortcut_hint.cancel()
        self.assertThat(self.unity.shortcut_hint.visible, Eventually(Equals(False)))

    def test_shortcut_hint_no_blur(self):
        """"""
        self.unity.shortcut_hint.ensure_visible()
        self.addCleanup(self.unity.shortcut_hint.ensure_hidden)

        self.assertThat(self.unity.shortcut_hint.get_shortcut_view().bg_texture_is_valid, Eventually(Equals(True)))


class ShortcutHintInteractionsTests(BaseShortcutHintTests):
    """Test the shortcuthint interactions with other Unity parts."""

    def test_shortcut_hint_hide_using_unity_shortcuts(self):
        """Unity shortcuts (like expo) must hide the shortcut hint."""
        self.unity.shortcut_hint.ensure_visible()
        self.addCleanup(self.unity.shortcut_hint.ensure_hidden)

        self.keybinding_tap("expo/start")
        self.addCleanup(self.keybinding, "expo/cancel")

    def test_shortcut_hint_hide_pressing_modifiers(self):
        """Pressing a modifer key must hide the shortcut hint."""
        self.unity.shortcut_hint.ensure_visible()
        self.addCleanup(self.unity.shortcut_hint.ensure_hidden)

        self.keyboard.press('Control')

        self.assertThat(self.unity.shortcut_hint.visible, Eventually(Equals(False)))

    def test_launcher_switcher_next_doesnt_show_shortcut_hint(self):
        """Super+Tab switcher cycling forward must not show shortcut hint."""
        switcher_timeout = self.unity.shortcut_hint.get_show_timeout()
        self.unity.shortcut_hint.show()
        self.addCleanup(self.unity.dash.ensure_hidden)
        self.addCleanup(self.unity.shortcut_hint.ensure_hidden)

        self.addCleanup(self.keyboard.press_and_release, "Escape")
        self.keybinding("launcher/switcher/next")
        self.keybinding("launcher/switcher/next")

        sleep(switcher_timeout * 2)

        self.assertThat(self.unity.shortcut_hint.visible, Equals(False))

        self.keybinding("launcher/switcher/prev")

    def test_launcher_switcher_prev_doesnt_show_shortcut_hint(self):
        """Super+Tab switcher cycling backwards must not show shortcut hint."""
        switcher_timeout = self.unity.shortcut_hint.get_show_timeout()
        self.unity.shortcut_hint.show()
        self.addCleanup(self.unity.dash.ensure_hidden)
        self.addCleanup(self.unity.shortcut_hint.ensure_hidden)

        self.addCleanup(self.keyboard.press_and_release, "Escape")
        self.keybinding("launcher/switcher/prev")
        self.keybinding("launcher/switcher/prev")

        sleep(switcher_timeout * 2)

        self.assertThat(self.unity.shortcut_hint.visible, Equals(False))

        self.keybinding("launcher/switcher/next")

    def test_launcher_icons_hints_show_with_shortcut_hint(self):
        """When the shortcut hint is shown also the launcer's icons hints should
        be shown.

        """
        launcher = self.get_launcher()
        self.unity.shortcut_hint.ensure_visible()
        self.addCleanup(self.unity.shortcut_hint.ensure_hidden)

        self.assertThat(self.unity.shortcut_hint.visible, Equals(True))
        self.assertThat(launcher.shortcuts_shown, Equals(True))

    def test_shortcut_hint_shows_with_launcher_icons_hints(self):
        """When the launcher icons hints are shown also the shortcut hint should
        be shown.

        """
        launcher = self.get_launcher()
        launcher.keyboard_reveal_launcher()
        self.addCleanup(launcher.keyboard_unreveal_launcher)

        self.assertThat(launcher.shortcuts_shown, Eventually(Equals(True)))
        self.assertThat(self.unity.shortcut_hint.visible, Eventually(Equals(True)))

    def test_shortcut_hint_closes_after_key_event(self):
        """ The shortcut hint must close when a key event comes through."""

        self.unity.shortcut_hint.show()
        self.assertThat(self.unity.shortcut_hint.visible, Eventually(Equals(True)))

        self.addCleanup(self.unity.shortcut_hint.ensure_hidden)

        self.keyboard.press_and_release("Ctrl")
        self.assertThat(self.unity.shortcut_hint.visible, Eventually(Equals(False)))



# This file is part of "blighty" and "uwidgets" which is released under GPL.
#
# See file LICENCE or go to http://www.gnu.org/licenses/ for full license
# details.
#
# uwidgets is a desktop widget creation and management library for Python 3.
#
# Copyright (c) 2018 Gabriele N. Tornetta <phoenix1987@gmail.com>.
# All rights reserved.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""Legacy visual tools.

Most of the classes in this module are inspired from Conky.
"""

from collections import deque


class Graph:
    """A uwidgets take on Conky graphsself.

    The constructor allows you to specify where the graph should be located
    as well as its size. You push values to it by calling the ``push_value``
    method. By default, the values are assumed to be in the range from 0 to
    100. If this is not the case, you can change the Y scale by specifying a
    value for the ``scale`` keyword argument.
    """
    def __init__(self, x, y, width, height, scale=100):
        self.x = x
        self.y = y
        self.width = width
        self.height = height
        self.scale = scale if scale else 0
        self.auto = not scale
        self._values = deque(maxlen=width)

    def push_value(self, v):
        self._values.append(v)
        if self.auto:
            self.scale = max(self._values)

    def draw(self, cr):
        if not self.scale:
            return

        offset = self.x + self.width - len(self._values)
        for i in range(len(self._values)):
            cr.set_line_width(1)
            if self.height > 0:
                cr.move_to(offset + i + .5, self.y + self.height)
                cr.line_to(
                    offset + i + .5,
                    self.y + int(self.height * (1-self._values[i]/self.scale))
                )
            else:
                cr.move_to(offset + i + .5, self.y)
                cr.line_to(
                    offset + i + .5,
                    self.y + int(-self.height * self._values[i] / self.scale)
                )
            cr.stroke()



# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
# Copyright 2012 Canonical
# Author: Marco Trevisan (Treviño)
#
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 3, as published
# by the Free Software Foundation.
#


import logging
from unity.emulators import UnityIntrospectionObject
from testtools.matchers import GreaterThan

from autopilot.introspection.types import Rectangle
from unity.emulators.dash import SearchBar

logger = logging.getLogger(__name__)


class Screen(UnityIntrospectionObject):
    """The Screen class."""

    @property
    def windows(self):
        """Return the available windows, or None."""
        return self.get_children_by_type(Window)

    @property
    def scaled_windows(self):
        """Return the available scaled windows, or None."""
        return self.get_children_by_type(Window, scaled=True)

    @property
    def spread_filter(self):
        """Return the spread filter, or None."""
        filter = self.get_children_by_type(SpreadFilter)
        if len(filter):
            return filter[0]

        return None

    def window(self, xid):
        """Return the window with given xid."""
        windows = self.get_children_by_type(Window, xid=xid)
        if len(windows):
            return windows[0]

        return None


class Window(UnityIntrospectionObject):
    """An individual window."""

    @property
    def geometry(self):
        """Returns a Rectangle (x,y,w,h) for the current window."""
        return self.globalRect

    @property
    def scale_close_geometry(self):
        """Returns a Rectangle (x,y,w,h) for the scale close button."""
        self.scaled_close_width.wait_for(GreaterThan(0))
        self.scaled_close_height.wait_for(GreaterThan(0))
        return Rectangle(self.scaled_close_x, self.scaled_close_y, self.scaled_close_width, self.scaled_close_height)


class SpreadFilter(UnityIntrospectionObject):
    """The spread filter."""

    @property
    def search_bar(self):
        """Return the search bar."""
        [search_bar] = self.get_children_by_type(SearchBar)
        return search_bar



# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
# Copyright 2012 Canonical
# Author: Thomi Richards
#
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 3, as published
# by the Free Software Foundation.
#


from autopilot.input import Keyboard, Mouse

from autopilot.keybindings import KeybindingsHelper
from testtools.matchers import GreaterThan

from unity.emulators.panel import WindowButtons

from unity.emulators import UnityIntrospectionObject
import logging
import dbus

logger = logging.getLogger(__name__)


class DashController(UnityIntrospectionObject, KeybindingsHelper):
    """The main dash controller object."""

    def __init__(self, *args, **kwargs):
        super(DashController, self).__init__(*args, **kwargs)
        self.keyboard = Keyboard.create()
        self.mouse = Mouse.create()

    def get_dash_view(self):
        """Get the dash view that's attached to this controller."""
        return self.get_children_by_type(DashView)[0]

    def hide_dash_via_dbus(self):
        """ Emulate a DBus call for dash hiding  """
        dash_object = dbus.SessionBus().get_object('com.canonical.Unity',
                                             '/com/canonical/Unity/Dash')
        dash_iface = dbus.Interface(dash_object, 'com.canonical.Unity.Dash')
        dash_iface.HideDash()

    @property
    def view(self):
        return self.get_dash_view()

    def toggle_reveal(self):
        """
        Reveals the dash if it's currently hidden, hides it otherwise.
        """
        old_state = self.visible
        logger.debug("Toggling dash visibility with Super key.")
        self.keybinding("dash/reveal", 0.1)
        self.visible.wait_for(not old_state)

    def ensure_visible(self, clear_search=True):
        """
        Ensures the dash is visible.
        """
        if not self.visible:
            self.toggle_reveal()
            self.visible.wait_for(True)
            if clear_search:
                self.clear_search()

    def ensure_hidden(self):
        """
        Ensures the dash is hidden.
        """
        if self.visible:
            self.hide_dash_via_dbus()
            self.visible.wait_for(False)

    @property
    def search_string(self):
        return self.searchbar.search_string

    @property
    def searchbar(self):
        """Returns the searchbar attached to the dash."""
        return self.view.get_searchbar()

    @property
    def preview_displaying(self):
        """Returns true if the dash is currently displaying a preview"""
        return self.view.preview_displaying;

    @property
    def preview_animation(self):
        """Returns the average progress of dash slip and animating a preview.
        Between 0.0 and 1.0.

        """
        return self.view.preview_animation;

    def get_num_rows(self):
        """Returns the number of displayed rows in the dash."""
        return self.view.num_rows

    def clear_search(self):
        """Clear the contents of the search bar.

        Assumes dash is already visible, and search bar has keyboard focus.

        """
        self.keyboard.press_and_release("Ctrl+a")
        self.keyboard.press_and_release("Delete")
        self.search_string.wait_for("")

    def reveal_application_scope(self, clear_search=True):
        """Reveal the application scope."""
        logger.debug("Revealing application scope with Super+a.")
        self._reveal_scope("lens_reveal/apps", clear_search)
        return self.view.get_scopeview_by_name("applications.scope")

    def reveal_music_scope(self, clear_search=True):
        """Reveal the music scope."""
        logger.debug("Revealing music scope with Super+m.")
        self._reveal_scope("lens_reveal/music", clear_search)
        return self.view.get_scopeview_by_name("music.scope")

    def reveal_file_scope(self, clear_search=True):
        """Reveal the file scope."""
        logger.debug("Revealing file scope with Super+f.")
        self._reveal_scope("lens_reveal/files", clear_search)
        return self.view.get_scopeview_by_name("files.scope")

    def reveal_video_scope(self, clear_search=True):
        """Reveal the video scope"""
        logger.debug("Revealing video scope with Super+v.")
        self._reveal_scope("lens_reveal/video", clear_search)
        return self.view.get_scopeview_by_name("video.scope")

    def reveal_command_scope(self, clear_search=True):
        """Reveal the 'run command' scope."""
        logger.debug("Revealing command scope with Alt+F2.")
        self._reveal_scope("lens_reveal/command", clear_search)
        return self.view.get_scopeview_by_name("commands.scope")

    def _reveal_scope(self, binding_name, clear_search):
        self.keybinding_hold(binding_name)
        self.keybinding_tap(binding_name)
        self.keybinding_release(binding_name)
        self.visible.wait_for(True)
        if clear_search:
            self.clear_search()

    @property
    def active_scope(self):
        return self.view.get_scopebar().active_scope

    def get_current_scope(self):
        """Get the currently-active ScopeView object."""
        active_scope_name = self.view.get_scopebar().active_scope
        return self.view.get_scopeview_by_name(active_scope_name)

    @property
    def geometry(self):
        return self.view.globalRect


class DashView(UnityIntrospectionObject):
    """The dash view."""

    def __get_window_buttons(self):
        """Return the overlay window buttons view."""
        buttons = self.get_children_by_type(OverlayWindowButtons)
        assert(len(buttons) == 1)
        return buttons[0]

    def get_searchbar(self):
        """Get the search bar attached to this dash view."""
        return self.get_children_by_type(SearchBar)[0]

    def get_scopebar(self):
        """Get the scopebar attached to this dash view."""
        return self.get_children_by_type(ScopeBar)[0]

    def get_scopeview_by_name(self, scope_name):
        """Get a ScopeView child object by it's name. For example, "home.scope"."""
        scopes = self.get_children_by_type(ScopeView)
        for scope in scopes:
            if scope.name == scope_name:
                return scope

    def get_preview_container(self):
        """Get the preview container attached to this dash view."""
        preview_containers = self.get_children_by_type(PreviewContainer)
        for preview_container in preview_containers:
            return preview_container
        return None

    @property
    def window_buttons(self):
        return self.__get_window_buttons().window_buttons()


class OverlayWindowButtons(UnityIntrospectionObject):
    """The Overlay window buttons class"""

    def window_buttons(self):
        buttons = self.get_children_by_type(WindowButtons)
        assert(len(buttons) == 1)
        return buttons[0]

class SearchBar(UnityIntrospectionObject):
    """The search bar for the dash view."""


class ScopeBar(UnityIntrospectionObject):
    """The bar of scope icons at the bottom of the dash."""
    def get_icon_by_name(self, name):
        """Get a ScopeBarIcon child object by it's name. For example, 'home.scope'."""
        icons = self.get_children_by_type(ScopeBarIcon)
        for icon in icons:
            if icon.name == name:
                return icon

class ScopeBarIcon(UnityIntrospectionObject):
    """A scope icon at the bottom of the dash."""


class ScopeView(UnityIntrospectionObject):
    """A Scope View."""

    def get_categories(self, only_visible=False):
        """Get a list of all groups within this scopeview. May return an empty list."""
        if only_visible:
            return self.get_children_by_type(PlacesGroup, is_visible=True)

        return self.get_children_by_type(PlacesGroup)

    def get_focused_category(self):
        """Return a PlacesGroup instance for the category whose header has keyboard focus.

        Returns None if no category headers have keyboard focus.

        """
        matches = self.get_children_by_type(PlacesGroup, header_has_keyfocus=True)
        return matches[0] if matches else None

    def get_category_by_name(self, category_name):
        """Return a PlacesGroup instance with the given name, or None."""
        matches = self.get_children_by_type(PlacesGroup, name=category_name)
        return matches[0] if matches else None

    def get_num_visible_categories(self):
        """Get the number of visible categories in this scope."""
        return len(self.get_categories(only_visible=True))

    def get_filterbar(self):
        """Get the filter bar for the current scope, or None if it doesn't have one."""
        bars = self.get_children_by_type(FilterBar)
        return bars[0] if bars else None


class PlacesGroup(UnityIntrospectionObject):
    """A category in the scope view."""

    def get_results(self):
        """Get a list of all results within this category. May return an empty list."""
        result_view = self.get_children_by_type(ResultView)[0]
        return result_view.get_children_by_type(Result)


class ResultView(UnityIntrospectionObject):
    """Contains a list of Result objects."""


class Result(UnityIntrospectionObject):
    """A single result in the dash."""

    def activate(self, double_click=True):
        m = Mouse.create()
        m.click_object(self, button=1)
        if double_click:
            m.click_object(self, button=1)

    def preview(self, button=1):
        Mouse.create().click_object(self, button)

    def preview_key(self):
        Mouse.create().move_to_object(self)

        k = Keyboard.create()
        k.press_and_release('Menu')

class FilterBar(UnityIntrospectionObject):
    """A filterbar, as shown inside a lens."""

    def get_num_filters(self):
        """Get the number of filters in this filter bar."""
        filters = self.get_children_by_type(FilterExpanderLabel)
        return len(filters)

    def get_focused_filter(self):
        """Returns the id of the focused filter widget."""
        filters = self.get_children_by_type(FilterExpanderLabel, expander_has_focus=True)
        return filters[0] if filters else None

    @property
    def expanded(self):
        """Return True if the filterbar on this scope is expanded, False otherwise.
        """
        searchbar = self._get_searchbar()
        return searchbar.showing_filters

    def ensure_expanded(self):
        """Expand the filter bar, if it's not already."""
        if not self.expanded:
            searchbar = self._get_searchbar()
            tx = searchbar.filter_label_x + (searchbar.filter_label_width / 2)
            ty = searchbar.filter_label_y + (searchbar.filter_label_height / 2)
            m = Mouse.create()
            m.move(tx, ty)
            m.click()
            self.expanded.wait_for(True)

    def ensure_collapsed(self):
        """Collapse the filter bar, if it's not already."""
        if self.expanded:
            searchbar = self._get_searchbar()
            tx = searchbar.filter_label_x + (searchbar.filter_label_width / 2)
            ty = searchbar.filter_label_y + (searchbar.filter_label_height / 2)
            m = Mouse.create()
            m.move(tx, ty)
            m.click()
            self.expanded.wait_for(False)

    def _get_searchbar(self):
        """Get the searchbar.

        This hack exists because there's now more than one SearchBar in Unity,
        and for some reason the FilterBar stuff is bundled in the SearchBar.

        """
        searchbar_state = self.get_state_by_path("//DashView/SearchBar")
        assert(len(searchbar_state) == 1)
        return self.make_introspection_object(searchbar_state[0])


class FilterExpanderLabel(UnityIntrospectionObject):
    """A label that expands into a filter within a filter bar."""

    def ensure_expanded(self):
        """Expand the filter expander label, if it's not already"""
        if not self.expanded:
            Mouse.create().click_object(self)
            self.expanded.wait_for(True)

    def ensure_collapsed(self):
        """Collapse the filter expander label, if it's not already"""
        if self.expanded:
            Mouse.create().click_object(self)
            self.expanded.wait_for(False)


class CoverArt(UnityIntrospectionObject):
    """A view which can be used to show a texture, or generate one using a thumbnailer."""


class RatingsButton(UnityIntrospectionObject):
    """A button which shows user rating as a function of stars."""


class Preview(UnityIntrospectionObject):
    """A preview of a dash scope result."""

    def get_num_actions(self):
        """Get the number of actions for the preview."""
        actions = self.get_children_by_type(ActionButton)
        return len(actions)

    def get_action_by_id(self, action_id):
        """Returns the action given it's action hint."""
        actions = self.get_children_by_type(ActionButton, action=action_id)
        return actions[0] if actions else None

    def execute_action_by_id(self, action_id):
        """Executes an action given by the id."""
        action = self.get_action_by_id(action_id)
        if action:
            Mouse.create().click_object(action)

    @property
    def cover_art(self):
        return self.get_children_by_type(CoverArt)

    @property
    def ratings_widget(self):
        return self.get_children_by_type(PreviewRatingsWidget)

    @property
    def info_hint_widget(self):
        return self.get_children_by_type(PreviewInfoHintWidget)

    @property
    def icon(self):
        return self.get_children_by_type(IconTexture)

    @property
    def text_boxes(self):
        return self.get_children_by_type(StaticCairoText)


class ApplicationPreview(Preview):
    """A application preview of a dash scope result."""

class GenericPreview(Preview):
    """A generic preview of a dash scope result."""

class MusicPreview(Preview):
    """A music preview of a dash scope result."""

class MoviePreview(Preview):
    """A movie preview of a dash scope result."""

class PreviewContent(UnityIntrospectionObject):
    """A preview content layout for the dash previews."""

    def get_current_preview(self):
        previews = self.get_children_by_type(Preview)
        if len(previews) > 0:
            return previews[0]
        return None


class PreviewContainer(UnityIntrospectionObject):
    """A container view for the main dash preview widget."""

    @property
    def content(self):
        return self.get_content()

    def get_num_previews(self):
        """Get the number of previews queued and current in the container."""
        previews = self.content.get_children_by_type(Preview)
        return len(previews)

    def get_content(self):
        """Get the preview content layout for the container."""
        return self.get_children_by_type(PreviewContent)[0]

    def get_left_navigator(self):
        """Return the left navigator object"""
        navigators = self.get_children_by_type(PreviewNavigator)
        for nav in navigators:
            if nav.direction == 2:
                return nav
        return None

    def get_right_navigator(self):
        """Return the right navigator object"""
        navigators = self.get_children_by_type(PreviewNavigator)
        for nav in navigators:
            if nav.direction == 3:
                return nav
        return None

    def navigate_left(self, count=1):
        """Navigate preview left"""
        m = Mouse.create()
        m.move_to_object(self.get_left_navigator().button_geo)

        old_preview_initiate_count = self.preview_initiate_count

        for i in range(count):
            self.navigate_left_enabled.wait_for(True)
            m.click()
            self.preview_initiate_count.wait_for(GreaterThan(old_preview_initiate_count))
            old_preview_initiate_count = self.preview_initiate_count

    def navigate_right(self, count=1):
        """Navigate preview right"""
        m = Mouse.create()
        m.move_to_object(self.get_right_navigator().button_geo)

        old_preview_initiate_count = self.preview_initiate_count

        for i in range(count):
            self.navigate_right_enabled.wait_for(True)
            m.click()
            self.preview_initiate_count.wait_for(GreaterThan(old_preview_initiate_count))
            old_preview_initiate_count = self.preview_initiate_count

    @property
    def animating(self):
        """Return True if the preview is animating, False otherwise."""
        return self.content.animating

    @property
    def waiting_preview(self):
        """Return True if waiting for a preview, False otherwise."""
        return self.content.waiting_preview

    @property
    def animation_progress(self):
        """Return the progress of the current preview animation."""
        return self.content.animation_progress

    @property
    def current_preview(self):
        """Return the current preview object."""
        return self.content.get_current_preview()
        preview_initiate_count_

    @property
    def preview_initiate_count(self):
        """Return the number of initiated previews since opened."""
        return self.content.preview_initiate_count

    @property
    def navigation_complete_count(self):
        """Return the number of completed previews since opened."""
        return self.content.navigation_complete_count

    @property
    def relative_nav_index(self):
        """Return the navigation position relative to the direction of movement."""
        return self.content.relative_nav_index

    @property
    def navigate_right_enabled(self):
        """Return True if right preview navigation is enabled, False otherwise."""
        return self.content.navigate_right_enabled

    @property
    def navigate_left_enabled(self):
        """Return True if left preview navigation is enabled, False otherwise."""
        return self.content.navigate_left_enabled


class PreviewNavigator(UnityIntrospectionObject):
    """A view containing a button to nagivate between previews."""

    def icon(self):
        return self.get_children_by_type(IconTexture);


class PreviewInfoHintWidget(UnityIntrospectionObject):
    """A view containing additional info for a preview."""


class PreviewRatingsWidget(UnityIntrospectionObject):
    """A view containing a rating button and user rating count."""


class Tracks(UnityIntrospectionObject):
    """A view containing music tracks."""


class Track(UnityIntrospectionObject):
    """A singular music track for dash prevews."""


class ActionButton(UnityIntrospectionObject):
    """A preview action button."""


class IconTexture(UnityIntrospectionObject):
    """An icon for the preview."""


class StaticCairoText(UnityIntrospectionObject):
    """Text boxes in the preview"""




# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
#
# Autopilot Functional Test Tool
# Copyright (C) 2012-2013 Canonical
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#


"""Functions to deal with ibus service."""


from gi.repository import IBus, GLib
import os
import logging
import subprocess


logger = logging.getLogger(__name__)


def get_ibus_bus():
    """Get the ibus bus object, possibly starting the ibus daemon if it's
    not already running.

    :raises: **RuntimeError** in the case of ibus-daemon being unavailable.

    """
    bus = IBus.Bus()
    if bus.is_connected():
        return bus

    main_loop = GLib.MainLoop()

    timeout = 5
    GLib.timeout_add_seconds(timeout, lambda *args: main_loop.quit())
    bus.connect("connected", lambda *args: main_loop.quit())

    os.spawnlp(os.P_NOWAIT, "ibus-daemon", "ibus-daemon", "--xim")

    main_loop.run()

    if not bus.is_connected():
        raise RuntimeError(
            "Could not start ibus-daemon after %d seconds." % (timeout))
    return bus


def get_available_input_engines():
    """Get a list of available input engines."""
    bus = get_ibus_bus()
    return [e.get_name() for e in bus.list_engines()]


def get_active_input_engines():
    """Get the list of input engines that have been activated."""
    bus = get_ibus_bus()
    return [e.get_name() for e in bus.list_active_engines()]


def set_active_engines(engine_list):
    """Installs the engines in *engine_list* into the list of active iBus
    engines.

    The specified engines must appear in the return list from
    get_available_input_engines().

    .. note:: This function removes all other engines.

    This function returns the list of engines installed before this function
    was called. The caller should pass this list to set_active_engines to
    restore ibus to it's old state once the test has finished.

    :param engine_list: List of engine names
    :type engine_list: List of strings
    :raises: **TypeError** on invalid *engine_list* parameter.
    :raises: **ValueError** when engine_list contains invalid engine name.

    """
    if type(engine_list) is not list:
        raise TypeError("engine_list must be a list of valid engine names.")
    available_engines = get_available_input_engines()
    for engine in engine_list:
        if not isinstance(engine, str):
            raise TypeError("Engines in engine_list must all be strings.")
        if engine not in available_engines:
            raise ValueError(
                "engine_list contains invalid engine name: '%s'", engine)

    bus = get_ibus_bus()
    config = bus.get_config()

    config.set_value("general",
                     "preload_engine_mode",
                     GLib.Variant.new_int32(IBus.PreloadEngineMode.USER))

    old_engines = get_active_input_engines()
    config.set_value(
        "general", "preload_engines", GLib.Variant("as", engine_list))

    return old_engines



# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
# Copyright 2012 Canonical
# Authors: Thomi Richards,
#          Marco Trevisan (Treviño)
#
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 3, as published
# by the Free Software Foundation.


from autopilot.matchers import Eventually
import logging
from testtools.matchers import Equals
from time import sleep

from unity.tests.launcher import LauncherTestCase

logger = logging.getLogger(__name__)


class LauncherShortcutTests(LauncherTestCase):
    """Tests for the shortcut hint window."""

    def setUp(self):
        super(LauncherShortcutTests, self).setUp()
        self.launcher_instance.keyboard_reveal_launcher()
        self.addCleanup(self.launcher_instance.keyboard_unreveal_launcher)
        sleep(2)

    def test_launcher_keyboard_reveal_shows_shortcut_hints(self):
        """Launcher icons must show shortcut hints after revealing with keyboard."""
        self.assertThat(self.launcher_instance.shortcuts_shown, Eventually(Equals(True)))

    def test_launcher_switcher_keeps_shorcuts(self):
        """Initiating launcher switcher after showing shortcuts must not hide shortcuts"""
        self.launcher_instance.switcher_start()
        self.addCleanup(self.launcher_instance.switcher_cancel)

        self.assertThat(self.unity.launcher.key_nav_is_active, Eventually(Equals(True)))
        self.assertThat(self.launcher_instance.shortcuts_shown, Eventually(Equals(True)))

    def test_launcher_switcher_next_keeps_shortcuts(self):
        """Launcher switcher next action must keep shortcuts after they've been shown."""
        self.launcher_instance.switcher_start()
        self.addCleanup(self.launcher_instance.switcher_cancel)
        self.launcher_instance.switcher_next()
        self.assertThat(self.launcher_instance.shortcuts_shown, Eventually(Equals(True)))

    def test_launcher_switcher_prev_keeps_shortcuts(self):
        """Launcher switcher prev action must keep shortcuts after they've been shown."""
        self.launcher_instance.switcher_start()
        self.addCleanup(self.launcher_instance.switcher_cancel)
        self.launcher_instance.switcher_prev()
        self.assertThat(self.launcher_instance.shortcuts_shown, Eventually(Equals(True)))

    def test_tooltip_not_shown(self):
        """Tooltip must not be shown after revealing the launcher with keyboard
        and mouse is not on the launcher.
        """
        self.assertThat(self.launcher_instance.tooltip_shown, Eventually(Equals(False)))



# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
# Copyright 2012 Canonical
# Author: Brandon Schaefer
#
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 3, as published
# by the Free Software Foundation.


from autopilot.matchers import Eventually
from testtools.matchers import Equals, GreaterThan
from time import sleep
import urllib.request, urllib.error, urllib.parse
import gettext

from unity.tests import UnityTestCase


class ShoppingScopeTests(UnityTestCase):
    """Test the shopping scope bahavior."""

    def setUp(self):
        super(ShoppingScopeTests, self).setUp()
        try:
            urllib.request.urlopen("http://www.google.com", timeout=2)
        except urllib.error.URLError as e:
            self.skip("Skipping test, no internet connection")
        gettext.install("unity-scope-shopping")

    def tearDown(self):
        self.unity.dash.ensure_hidden()
        super(ShoppingScopeTests, self).tearDown()

    def test_no_results_in_home_scope_if_empty_search(self):
        """Test that the home scope contains no results if the search bar is empty."""
        self.unity.dash.ensure_visible()
        scope = self.unity.dash.get_current_scope()

        results_category = scope.get_category_by_name(_("More suggestions"))
        refresh_results_fn = lambda: len(results_category.get_results())
        self.assertThat(refresh_results_fn, Eventually(Equals(0)))

    def test_home_scope_has_shopping_results(self):
        """Test that the home scope contains results."""
        self.unity.dash.ensure_visible()
        scope = self.unity.dash.get_current_scope()

        self.keyboard.type("playstation")
        results_category = scope.get_category_by_name(_("More suggestions"))

        refresh_results_fn = lambda: len(results_category.get_results())
        self.assertThat(refresh_results_fn, Eventually(GreaterThan(1), timeout=25))

    def test_application_scope_has_shopping_results(self):
        """Test that the application scope contains results."""
        self.unity.dash.reveal_application_scope()
        scope = self.unity.dash.get_current_scope()

        self.keyboard.type("Text Editor")
        results_category = scope.get_category_by_name(_("More suggestions"))

        refresh_results_fn = lambda: len(results_category.get_results())
        self.assertThat(refresh_results_fn, Eventually(GreaterThan(1), timeout=25))

    def test_music_scope_has_shopping_results(self):
        """Test that the music scope contains results."""
        self.unity.dash.reveal_music_scope()
        scope = self.unity.dash.get_current_scope()

        self.keyboard.type("megadeth")
        results_category = scope.get_category_by_name(_("More suggestions"))

        refresh_results_fn = lambda: len(results_category.get_results())
        self.assertThat(refresh_results_fn, Eventually(GreaterThan(1), timeout=25))

    def test_preview_works_with_shopping_scope(self):
        """This test shows the dash preview works with shopping scope results."""
        self.unity.dash.ensure_visible()
        scope = self.unity.dash.get_current_scope()

        self.keyboard.type("playstation")
        results_category = scope.get_category_by_name(_("More suggestions"))

        refresh_results_fn = lambda: len(results_category.get_results())
        self.assertThat(refresh_results_fn, Eventually(GreaterThan(1), timeout=25))

        results = results_category.get_results()
        results[0].preview()

        self.assertThat(self.unity.dash.preview_displaying, Eventually(Equals(True)))

    def test_shopping_scope_preview_navigate_right(self):
        """This test shows that shopping scope results can open previews,
        then move to the next shopping result.
        """
        self.unity.dash.ensure_visible()
        scope = self.unity.dash.get_current_scope()

        self.keyboard.type("playstation")
        results_category = scope.get_category_by_name(_("More suggestions"))

        refresh_results_fn = lambda: len(results_category.get_results())
        self.assertThat(refresh_results_fn, Eventually(GreaterThan(2), timeout=25))

        results = results_category.get_results()
        results[0].preview()

        self.assertThat(self.unity.dash.preview_displaying, Eventually(Equals(True)))
        self.preview_container = self.unity.dash.view.get_preview_container()
        start_index = self.preview_container.relative_nav_index
        self.preview_container.navigate_right()

        self.assertThat(self.preview_container.relative_nav_index, Eventually(Equals(start_index+1)))



# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
# Copyright 2012 Canonical
# Authors: Thomi Richards,
#          Marco Trevisan (Treviño)
#
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 3, as published
# by the Free Software Foundation.


from autopilot.matchers import Eventually
import logging
from testtools.matchers import Equals
from time import sleep

from unity.emulators.icons import BFBLauncherIcon
from unity.tests.launcher import LauncherTestCase

logger = logging.getLogger(__name__)


class LauncherVisualTests(LauncherTestCase):
    """Tests for visual aspects of the launcher (icon saturation etc.)."""

    def test_keynav_from_dash_saturates_icons(self):
        """Starting super+tab switcher from the dash must resaturate launcher icons.

        Tests fix for bug #913569.
        """
        bfb = self.unity.launcher.model.get_bfb_icon()
        self.mouse.move(bfb.center.x, bfb.center.y)
        self.unity.dash.ensure_visible()
        current_monitor = self.unity.dash.monitor

        sleep(1)
        # We can't use 'launcher_instance.switcher_start()' since it moves the mouse.
        self.keybinding_hold_part_then_tap("launcher/switcher")
        self.addCleanup(self.keybinding_release, "launcher/switcher")
        self.addCleanup(self.keybinding, "launcher/switcher/exit")

        self.keybinding_tap("launcher/switcher/next")
        for icon in self.unity.launcher.model.get_launcher_icons():
            self.assertFalse(icon.monitors_desaturated[current_monitor])

    def test_opening_dash_desaturates_icons(self):
        """Opening the dash must desaturate all the launcher icons."""
        self.unity.dash.ensure_visible()
        current_monitor = self.unity.dash.monitor

        self.addCleanup(self.unity.dash.ensure_hidden)

        for icon in self.unity.launcher.model.get_launcher_icons():
            if isinstance(icon, BFBLauncherIcon):
                self.assertFalse(icon.monitors_desaturated[current_monitor])
            else:
                self.assertTrue(icon.monitors_desaturated[current_monitor])

    def test_opening_dash_with_mouse_over_launcher_keeps_icon_saturation(self):
        """Opening dash with mouse over launcher must not desaturate icons."""
        launcher_instance = self.get_launcher()
        x,y,w,h = launcher_instance.geometry
        self.mouse.move(x + w/2, y + h/2)
        sleep(.5)
        self.unity.dash.ensure_visible()
        current_monitor = self.unity.dash.monitor

        self.addCleanup(self.unity.dash.ensure_hidden)
        for icon in self.unity.launcher.model.get_launcher_icons():
            self.assertFalse(icon.monitors_desaturated[current_monitor])

    def test_mouse_over_with_dash_open_desaturates_icons(self):
        """Moving mouse over launcher with dash open must saturate icons."""
        self.unity.dash.ensure_visible()
        current_monitor = self.unity.dash.monitor

        self.addCleanup(self.unity.dash.ensure_hidden)
        sleep(.5)
        self.mouse.move_to_object(self.get_launcher())
        sleep(.5)
        for icon in self.unity.launcher.model.get_launcher_icons():
            self.assertFalse(icon.monitors_desaturated[current_monitor])



# This file is part of "blighty" and "uwidgets" which is released under GPL.
#
# See file LICENCE or go to http://www.gnu.org/licenses/ for full license
# details.
#
# uwidgets is a desktop widget creation and management library for Python 3.
#
# Copyright (c) 2018 Gabriele N. Tornetta <phoenix1987@gmail.com>.
# All rights reserved.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""
This module provides support for creating X11 canvases. If you are trying to
replicate conky's behaviour, the API offered by this module is the closest to
it.
"""

from uwidgets._x11 import *

from . canvas import Canvas
